from .eager import *
from .surface import CustomSurface
from sparrow.file_ops import yaml_load, ppath


_config =  yaml_load(ppath("version_config.yaml"))
__version__ = _config["version"]
print(f"{_config['name']} version: {__version__}")



from manimlib import *
from .tools import calc_number_step

def m_line(x,
           y,
           z=None,
           axes=None,
           color=None,
           width=None,
           optacity=None,
           background=None):
    """point data to line"""
    L = len(x)
    x, y = np.array(x), np.array(y)
    if z is None:
        z = np.zeros(x.shape)
    if axes is None:
        # last_little_line = Line([x[L - 1], y[L - 1], z[0]], [x[0], y[0], z[0]])
        line = VGroup(*[
            Line([x[i], y[i], 0], [x[i + 1], y[i + 1]]).set_stroke(
                color=color,
                width=width,
                opacity=optacity,
                background=background) for i in range(len(x) - 1)
        ])
    elif type(axes) is Axes:
        # last_little_line = Line(axes.c2p(x[L - 1], y[L - 1]),
        #                         axes.c2p(x[0], y[0]))
        line = VGroup(*[
            Line(axes.c2p(x[i], y[i]), axes.c2p(x[i + 1], y[
                i + 1])).set_stroke(color=color,
                                    width=width,
                                    opacity=optacity,
                                    background=background)
            for i in range(len(x) - 1)
        ])
    else:  # ThreeDAxes
        # last_little_line = Line(axes.c2p(x[L - 1], y[L - 1], z[0]),
        #                         axes.c2p(x[0], y[0], z[0])).set_stroke(
        #     color=color, width=width, opacity=optacity, background=background)

        line = VGroup(*[
            Line(axes.c2p(x[i], y[i], z[i]), axes.c2p(x[i + 1], y[
                i + 1], z[i])).set_stroke(color=color,
                                          width=width,
                                          opacity=optacity,
                                          background=background)
            for i in range(len(x) - 1)
        ])
    return line


def m_scatter(x, y, z=None, axes=None):
    """for scatter plot"""
    x, y = np.array(x), np.array(y)
    if z is None:
        z = np.zeros(x.shape)
    dots = VGroup(
        *
        [Dot(radius=.01).move_to([xi, yi, zi]) for xi, yi, zi in zip(x, y, z)])

    return dots


class Plot:
    def __init__(self):
        self.num = 0
        self._axes: Axes = None
        self._axes_labels = []
        self._xmin = np.Inf
        self._xmax = -np.inf
        self._ymin = np.Inf
        self._ymax = -np.inf
        self._unit_x = 0
        self._unit_y = 0
        self._xdata = []
        self._ydata = []
        self._color_list = []
        self._color_choice_list = [
            GREEN_C, BLUE_C, RED_C, YELLOW_C, ORANGE, GOLD_C, MAROON_C, TEAL_C
        ]
        self._width_list = []
        self._axes_line_list = []
        self._axes_width = 10
        self._axes_height = 6.2
        self._axes_ratio = 0.62
        self._scale_ratio = None
        self._num_decimal_places = (1, 1)
        self._show_axes = True
        self._include_tip = True
        self._x_label = 'x'
        self._y_label = 'y'

    def create_axes(self, x_label_min, x_label_max, y_label_min, y_label_max):
        x_length = x_label_max - x_label_min
        y_length = y_label_max - y_label_min
        dx = x_length / 20
        dy = y_length / 15

        xmin, xmax = x_label_min - EPSILON, x_label_max + dx
        ymin, ymax = y_label_min - EPSILON, y_label_max + dy

        if self._scale_ratio is not None:
            tick_ratio = self._scale_ratio * y_length/ x_length
        else:
            tick_ratio = self._axes_ratio

        self._axes_height = self._axes_width * tick_ratio

        if self._axes_height > 7:
            self._axes_height = 7
            self._axes_width = self._axes_height / tick_ratio

        # n_x = 10
        # n_y = n_x * tick_ratio
        # x_step = x_length / n_x
        # y_step = y_length / n_y
        n_step_x, x_step = calc_number_step(x_length)
        n_step_y, y_step = calc_number_step(y_length)

        if n_step_y / n_step_x > tick_ratio + 0.3:
            y_step *= 2

        axes = Axes(
            x_range=(xmin, xmax, x_step),
            y_range=(y_label_min, ymax, y_step),
            height=self._axes_height,
            width=self._axes_width,
            # Axes is made of two NumberLine mobjects.  You can specify
            # their configuration with axis_config
            axis_config={
                "numbers_to_exclude": [],
                "stroke_color": GREY_A,
                "stroke_width": 0.3,
                "include_tip": self._include_tip,
            },
            # Alternatively, you can specify configuration for just one
            # of them, like this.
            x_axis_config={
                "tip_config": {"width": 0.1, "length": 0.25},
                "decimal_number_config": {
                    "num_decimal_places": self._num_decimal_places[0],
                    "font_size": 15,
                },
            },
            y_axis_config={
                "tip_config": {"width": 0.15, "length": 0.3},
                "decimal_number_config": {
                    "num_decimal_places": self._num_decimal_places[1],
                    "font_size": 15,
                },
            },

        )

        axes.add_coordinate_labels()

        self._unit_y = axes.c2p(0, 1)[1] - axes.c2p(0, 0)[1]
        self._unit_x = axes.c2p(1, 0)[0] - axes.c2p(0, 0)[0]

        axes.x_axis.shift(UP * self._unit_y * y_label_min)
        axes.y_axis.shift(RIGHT * self._unit_x * x_label_min)

        self._axes = axes

    def get_axes(self):
        return self._axes

    def get_axes_lines(self):
        return {"line": self._axes_line_list, "axes": self._axes_labels}

    def gen_axes_lines(self):
        self.create_axes(x_label_min=self._xmin,
                         x_label_max=self._xmax,
                         y_label_min=self._ymin,
                         y_label_max=self._ymax)
        # midx = (self._xmin + self._xmax) / 2
        # midy = (self._ymin + self._ymax) / 2
        # frame = scene.camera.frame
        # frame.move_to(self._axes.c2p(midx, midy))
        for x, y, color, width in zip(self._xdata, self._ydata,
                                      self._color_list, self._width_list):
            line = m_line(x, y, color=color, width=width, axes=self._axes)

            labels = VGroup(
                self._axes.get_axis_label(self._x_label,
                                          self._axes.get_x_axis(),
                                          edge=RIGHT,
                                          direction=RIGHT).scale(0.8),
                self._axes.get_axis_label(self._y_label,
                                          self._axes.get_y_axis(),
                                          edge=UP,
                                          direction=UP).scale(0.8),
            )
            self._axes_labels = VGroup(self._axes,
                                       labels) if self._show_axes else []
            self._axes_line_list.append(line)

            line.shift(-self._unit_x * self._xmin * RIGHT)
            # self._axes_line_list.append(line)

    def plot(
        self,
        x,
        y,
        color=None,
        width=None,
        axes_ratio=0.618,
        scale_ratio=None,
        show_axes=True,
        include_tip=True,
        num_decimal_places=(1, 1),
        x_label='x',
        y_label='y',
    ):
        self.num += 1
        self._show_axes = show_axes
        self._xmin = min(self._xmin, min(x))
        self._xmax = max(self._xmax, max(x))
        self._ymin = min(self._ymin, min(y))
        self._ymax = max(self._ymax, max(y))

        self._xdata.append(x)
        self._ydata.append(y)
        if color is None:
            color = self._color_choice_list[self.num %
                                            len(self._color_choice_list) - 1]
        self._color_list.append(color)
        self._width_list.append(width)

        self._num_decimal_places = num_decimal_places
        self._axes_ratio = axes_ratio
        self._scale_ratio = scale_ratio
        self._include_tip = include_tip
        self._x_label = x_label
        self._y_label = y_label



import os
import sys
from sparrow.file_ops import path


def ppath(string, file=__file__) -> str:
    """Path in package"""
    return path(os.path.join(os.path.dirname(file), string))


def calc_number_digit(number):
    res = number
    digit = 0
    if res >= 1:
        while res > 10:
            digit += 1
            # res, mod = np.divmod(res, 10)
            res //= 10
    else:
        while res < 1:
            digit -= 1
            res *= 10
    return 10**digit


def calc_number_step(number):
    digit = calc_number_digit(number)
    n = number / digit
    while True:
        if n < 5:
            digit /= 5
            n = round(number / digit)
        elif n > 10:
            digit *= 2
            n = round(number / digit)
        else:
            break
    return n, digit



from manimlib import *
from my_geometry import *

## test Arcs
class Arcs_Test(Scene):

    def construct(self):

        arcs_01 = Arcs(stroke_width=80).shift(LEFT * 4.5)
        arcs_02 = Arcs(angle_list=np.array([10, 20, 30, 40, 50, 60, 70, 80]) * DEGREES, stroke_width=200)
        arcs_03 = Arcs(angle_list=np.array([10, 15, 20, 30]) * DEGREES, stroke_width=200).set_stroke(opacity=0.25).shift(RIGHT * 4)
        arcs_04 = Arcs(angle_list=np.array([10, 15, 20, 30]) * DEGREES, radius=2, stroke_width=10).shift(RIGHT * 4)

        self.play(ShowCreation(arcs_01))
        self.wait()
        self.play(ShowCreation(arcs_02))
        self.wait()
        self.play(ShowCreation(VGroup(arcs_03, arcs_04)))

        self.wait(4)

## test Angle
class Angle_test(Scene):

    def construct(self):

        A = LEFT * 4.5 + DOWN * 2
        B = RIGHT * 6 + DOWN * 1
        C = UP * 2

        tri_abc = Polygon(A, B, C, color=WHITE)

        dot_A = Dot(A, color=RED, radius=0.15)
        angle_A = Angle(B, A, C, color=RED, radius=1.6)

        dot_B = Dot(B, color=YELLOW, radius=0.15)
        angle_B = Angle(A, B, C, color=YELLOW, radius=1.5)

        dot_C = Dot(C, color=BLUE, radius=0.15)
        angle_C = Angle(A, C, B, color=BLUE, radius=1.)

        self.add((tri_abc))
        self.wait()
        self.play(FadeIn(dot_A))
        self.play(ShowCreation(angle_A))
        self.wait()
        self.play(FadeIn(dot_B))
        self.play(ShowCreation(angle_B))
        self.wait()
        self.play(FadeIn(dot_C))
        self.play(ShowCreation(angle_C))

        self.wait(2)

## test Tracked_Point 01
class Test_Tracked_Point(Scene):

    def construct(self):
        numberplane = NumberPlane()
        self.play(ShowCreation(numberplane))
        self.wait()
        point = Tracked_Point(RIGHT * 3, size=0.25)

        self.play(FadeIn(point), ShowCreation(point.coordinates_text))
        self.wait()
        self.play(Rotating(point, radians=TAU, about_point=ORIGIN), run_time=10)
        self.wait(2)

## test Tracked_Point 02
class Point_move_along_sinX(Scene):

    def construct(self):

        numberplane = NumberPlane()

        path = ParametricCurve(lambda t: np.sin(t*PI/2) * UP + t * RIGHT,t_range=(-2, 2),
                                color=PINK)

        point = Tracked_Point(LEFT * 2, size=0.2)

        self.add(numberplane)
        self.play(ShowCreation(path))
        self.wait()
        self.play(ShowCreation(point))
        self.wait()
        self.add(point.coordinates_text)
        self.play(MoveAlongPath(point, path, rate_func=linear), run_time=5)
        self.wait(2)

## test Right_angle and Dashed_Circle
class Test_Right_Angle(Scene):

    def construct(self):

        cp = ComplexPlane().scale(2.4)

        arrow_01 = Arrow(cp.n2p(1), cp.n2p(0.5), color=BLUE, buff=0, plot_depth=1)
        arrow_02 = Arrow(cp.n2p(1), cp.n2p(1+0.5j), color=YELLOW, buff=0, plot_depth=1)
        dot = Dot(cp.n2p(1), color=GREEN, plot_depth=2)
        group_01 = VGroup(dot, arrow_01, arrow_02)
        ra = Right_angle(corner=dot.get_center(), on_the_right=False)

        # the Right_angle 'ra' will not rotate with group_01,
        # but use method 'move_corner_to' & 'change_angle_to' to adjust its position and attitude
        ra.add_updater(lambda ra: ra.move_corner_to(dot.get_center()))
        ra.add_updater(lambda ra: ra.change_angle_to(arrow_01.get_angle() + PI))

        dash_circle = Dashed_Circle(radius=cp.n2p(1)[0], arc_config={'color': GREEN, 'stroke_width': 1.5})

        self.play(ShowCreation(cp))
        self.wait()
        self.play(ShowCreation(dot))
        self.play(ShowCreation(arrow_01), ShowCreation(arrow_02))
        self.play(ShowCreation(ra))
        self.wait()
        self.play(ShowCreation(dash_circle))

        self.play(Rotating(group_01, about_point=ORIGIN))

        self.wait(2)

## test my_text ##
from my_text import MyText
class Test_mytext(Scene):

    def construct(self):

        color_dict = {'R': PINK, 'd': YELLOW, 'r': ORANGE, '\\theta': BLUE, '\\over': WHITE,
              't': BLUE, 'e': GREEN, 'i': RED, '\\sin': WHITE, '\\cos': WHITE}

        font_list = ['Comic Sans MS', '庞门正道标题体', 'Consolas', 'SWGothe', 'Rough___Dusty_Chalk',
                     'SWScrps', '新蒂小丸子体']

        origin_formula = Tex('f', '(', 't', ')', '=', 'x', '(', 't', ')', '+', 'y', '(', 't', ')', 'i', '=',
                             '(', 'R', '-', 'r', ')', 'e^{', 'i', 't}', '+', 'd', 'e^{', '-', 'i', '{R', '-',
                             'r', '\\over', 'r}', 't}').scale(1)\
                        .set_color_by_tex_to_color_map(color_dict).to_corner(LEFT * 2 + UP * 1.5)
        formulas = VGroup(origin_formula)

        for i in range(len(font_list)):
            formula_i = MyText('f', '(', 't', ')', '=', 'x', '(', 't', ')', '+', 'y', '(', 't', ')', 'i', '=',
                             '(', 'R', '-', 'r', ')', 'e^{', 'i', 't}', '+', 'd', 'e^{', '-', 'i', '{R', '-',
                             'r', '\\over', 'r}', 't}', default_font=font_list[i], tex_scale_factor=0.75)
            formula_i.set_color_by_tex_to_color_map(color_dict)
            replace_dict = {'e^{': 'e', 't}': 't', '{R': 'R', 'r}': 'r', '\\over': '-'}
            new_formula = formula_i.get_new_font_texs(replace_dict)
            new_formula.to_corner(LEFT * 2 + UP * 1.5).shift(DOWN * 0.8 * (i+1))
            formulas.add(new_formula)

        self.add(formulas)
        self.wait(5)

# class Test_P(Scene):
#
#     CONFIG = {
#         'camera_config':{
#             'background_color': WHITE,
#         },
#     }
#
#     def construct(self):
#
#         text_1 = PStyle_Text('Xg', 'nb').to_corner(LEFT * 2 + UP * 2)
#         text_2 = PStyle_Text('Manim', 'Kindergarten').to_corner(LEFT * 2 + UP * 5)
#         text_3 = PStyle_Text('Cigar', '666').to_corner(LEFT * 2 + UP * 8)
#         text_4 = PStyle_Text('Bili', 'Bili').to_corner(LEFT * 2 + UP * 11)
#
#         text_2[-1].shift(DOWN * 0.064)
#         text_2[0].shift(DOWN * 0.04)
#         text_1[-1].shift(UP * 0.02)
#         text_1[0].shift(UP * 0.01)
#
#         self.add(text_1, text_2, text_3, text_4)
#
#         self.wait(2)

## test Trail ##

class Test_trail(Scene):

    def construct(self):

        dot = Dot(color=BLUE).shift(LEFT)
        trail = Trail(dot, trail_color=BLUE_B, max_width=4)
        trail.start_trace()

        self.add(trail)
        self.play(Rotating(dot, about_point=ORIGIN, run_time=4))

        trail.retrieve_trail(rate=5)

        self.wait(2)

class Test_trail_02(Scene):

    def construct(self):

        poly = RegularPolygon(5).scale(1.6)
        dot = Dot().set_fill(opacity=0).move_to(poly.get_start())
        trail_dot = Trail(dot, rate_func=lambda t: 1, trail_color=[RED, ORANGE, YELLOW, GREEN, BLUE, PINK, RED],
                          nums=300, max_width=2.5)
        trail_dot.start_trace()
        self.add(trail_dot.trail)
        self.play(MoveAlongPath(dot, poly), run_time=10)
        self.wait(4)

## test MySectors by data visualisation ##

class Test_MySectors(Scene):

    CONFIG = {
        'camera_config':{
            'background_color': WHITE,
        },
    }

    def construct(self):

        values = [2, 2, 2, 3, 9, 9, 10, 11, 14, 15, 15, 15, 15, 16, 18, 18, 18,
                  19, 22, 23, 23, 24, 24, 25, 26, 37, 44]
        labels = ['广东', '山东', '河南', '黑龙江', '四川', '浙江', '宁夏', '辽宁',
                  '湖南', '天津', '河北', '江西', '安徽', '福建', '山西', '广西', '重庆',
                  '吉林', '云南', '海南', '陕西', '内蒙古', '江苏', '新疆', '贵州', '青海', '西藏']

        center = UP * 0.5 + LEFT * 2.25

        graph_01 = MySectors(inner_radius=1.5, values=values, labels=labels, start_direction=RIGHT,
                             unit='天', center=center)
        color = average_color(BLUE_C, BLACK, BLACK)
        graph_01.create_cicles(color)
        graph_01.create_circle_shadow(color=color)

        font='华光标题宋_CNKI'
        text_01 = Text('多个省市区', font=font, color=color, size=0.32)
        text_02 = Text('确诊病例连续多日', font=font, color=color, size=0.32).next_to(text_01, DOWN * 0.25)
        text_03 = Text('零新增', font=font, color=color, size=0.75).next_to(text_02, DOWN * 0.4)
        texts = VGroup(text_01, text_02, text_03).move_to(center)

        rect_1 = Rectangle(width=0.1, height=0.24, stroke_width=0, fill_color=color, fill_opacity=1)
        text_11 = Text('数据为3月14日前无新增确诊病例的27个省（市，区）', font='思源黑体 Bold', color=color).set_height(0.24).next_to(rect_1, RIGHT * 0.32).align_to(rect_1, DOWN)
        text_11.set_color_by_t2c({'3月14日': BLUE})
        text_line_01 = VGroup(rect_1, text_11).to_corner(LEFT * 16 + UP * 1.)
        rect_2 = Rectangle(width=0.1, height=0.24, stroke_width=0, fill_color=color, fill_opacity=1)
        text_12 = Text('数据来源：@央视新闻', font='思源黑体 Bold', color=color).set_height(0.24).next_to(rect_2, RIGHT * 0.32).align_to(rect_2, DOWN)
        text_12.set_color_by_t2c({'@央视新闻': ORANGE})
        text_line_02 = VGroup(rect_2, text_12).to_corner(LEFT * 16 + UP * 1.75)
        rect_3 = Rectangle(width=0.1, height=0.24, stroke_width=0, fill_color=color, fill_opacity=1)
        text_13 = Text('作者：@cigar666', font='思源黑体 Bold', color=color).set_height(0.24).next_to(rect_3, RIGHT * 0.32).align_to(rect_3, DOWN)
        text_13.set_color_by_t2c({'@cigar666': PINK})
        text_line_03 = VGroup(rect_3, text_13).to_corner(LEFT * 16 + UP * 2.5)

        self.add(graph_01, texts, text_line_01, text_line_02, text_line_03)
        self.wait(5)

class Test_MySectors_0315(Scene):

    CONFIG = {
        'camera_config':{
            'background_color': WHITE,
        },
    }

    def construct(self):

        values = [3, 3, 4, 10, 11, 12, 15, 16, 16, 16, 16, 17, 19, 19, 19,
                  20, 23, 24, 24, 25, 25, 26, 27, 38, 45]
        labels = ['山东', '河南', '黑龙江', '四川', '宁夏', '辽宁',
                  '湖南', '天津', '河北', '江西', '安徽', '福建', '山西', '广西', '重庆',
                  '吉林', '云南', '海南', '陕西', '内蒙古', '江苏', '新疆', '贵州', '青海', '西藏']

        center = UP * 0.5 + LEFT * 2.25

        graph_01 = MySectors(inner_radius=1.5, values=values, labels=labels, start_direction=RIGHT,
                             unit='天', center=center)
        color = average_color(BLUE_B, BLACK, BLACK)
        graph_01.create_cicles(color)
        graph_01.create_circle_shadow(color=color)

        font='华光标题宋_CNKI'
        text_01 = Text('多个省市区', font=font, color=color, size=0.32)
        text_02 = Text('确诊病例连续多日', font=font, color=color, size=0.32).next_to(text_01, DOWN * 0.25)
        text_03 = Text('零新增', font=font, color=color, size=0.75).next_to(text_02, DOWN * 0.4)
        texts = VGroup(text_01, text_02, text_03).move_to(center)

        rect_1 = Rectangle(width=0.1, height=0.24, stroke_width=0, fill_color=color, fill_opacity=1)
        text_11 = Text('数据为3月15日前无新增确诊病例的25个省（市，区）', font='思源黑体 Bold', color=color).set_height(0.24).next_to(rect_1, RIGHT * 0.32).align_to(rect_1, DOWN)
        text_11.set_color_by_t2c({'3月15日': BLUE})
        text_line_01 = VGroup(rect_1, text_11).to_corner(LEFT * 16 + UP * 1.)
        rect_2 = Rectangle(width=0.1, height=0.24, stroke_width=0, fill_color=color, fill_opacity=1)
        text_12 = Text('数据来源：@央视新闻', font='思源黑体 Bold', color=color).set_height(0.24).next_to(rect_2, RIGHT * 0.32).align_to(rect_2, DOWN)
        text_12.set_color_by_t2c({'@央视新闻': ORANGE})
        text_line_02 = VGroup(rect_2, text_12).to_corner(LEFT * 16 + UP * 1.75)
        rect_3 = Rectangle(width=0.1, height=0.24, stroke_width=0, fill_color=color, fill_opacity=1)
        text_13 = Text('作者：@cigar666', font='思源黑体 Bold', color=color).set_height(0.24).next_to(rect_3, RIGHT * 0.32).align_to(rect_3, DOWN)
        text_13.set_color_by_t2c({'@cigar666': PINK})
        text_line_03 = VGroup(rect_3, text_13).to_corner(LEFT * 16 + UP * 2.5)

        self.add(graph_01, texts, text_line_01, text_line_02, text_line_03)
        self.wait(5)

## test New_Polygon ##

class Test_New_Polygon(Scene):

    def construct(self):

        tri_01 = New_Polygon(ORIGIN, 2.5*UP, 5*RIGHT, color=BLUE, stroke_width=20).shift(LEFT * 5.5)
        tri_02 = Polygon(ORIGIN, 2.5*UP, 5*RIGHT, color=GREEN, stroke_width=20).shift(RIGHT * 0.5)

        text_01 = Text('New_Polygon', font='思源黑体 Bold', color=BLUE)\
            .set_height(0.5).next_to(tri_01, DOWN * 0.75)
        text_02 = Text('Polygon', font='思源黑体 Bold', color=GREEN)\
            .set_height(0.5).next_to(tri_02, DOWN * 0.75)

        self.add(*tri_01, tri_02, text_01, text_02)
        self.play(tri_01[0].set_fill, {'color': YELLOW, 'opacity': 0.6},
                  tri_02.set_fill, {'color': YELLOW, 'opacity': 0.6}, run_time=2.5)
        self.wait(2)

class Test_New_Polygon_02(Scene):

    def construct(self):

        tri_01 = New_Polygon(ORIGIN, 2.5*UP, 5*RIGHT, color=BLUE, stroke_width=30).move_to(ORIGIN)

        self.add(*tri_01)
        self.wait(0.5)
        self.play(tri_01.scale, 2., run_time=1.5)
        self.wait()
        self.play(tri_01[0].set_fill, {'color': YELLOW, 'opacity': 0.6}, run_time=1.5)
        self.wait(2)


## test Shadow ##

class Test_Shadow(Scene):
    CONFIG = {
        'camera_config': {
            'background_color': WHITE,
        },
    }
    def construct(xgnb):
        # circle = Circle(color=BLUE, stroke_width=0)
        text = Text('X G N B !', font='庞门正道标题体', size=1.8)
        shadow = Shadow_2d(text, blur_width=0.5, layer_num=50, show_basic_shape=False, shadow_out=True).shift(UP * 2.5)
        shadow_02 = VGroup(*[Shadow_2d(text[2 * i], blur_width=0.5, layer_num=50, show_basic_shape=False) for i in range(5)]).shift(DOWN * 2)

        path = 'my_manim_projects\\my_projects\\resource\\svg_files\\'
        good = SVGMobject(path + 'good.svg', color=PINK).to_corner(LEFT * 5 + DOWN * 2)
        coin = SVGMobject(path + 'coin.svg', color=BLUE).to_corner(LEFT * 12 + DOWN * 2)
        favo = SVGMobject(path + 'favo.svg', color=ORANGE).to_corner(LEFT * 19 + DOWN * 2)

        shadow_good = Shadow_2d(good, blur_width=0.3, layer_num=50, show_basic_shape=False, shadow_out=True)
        shadow_coin = Shadow_2d(coin, blur_width=0.3, layer_num=50, show_basic_shape=False, shadow_out=True)
        shadow_favo = Shadow_2d(favo, blur_width=0.3, layer_num=50, show_basic_shape=False, shadow_out=True)

        xgnb.add(shadow, shadow_02)
        xgnb.add(shadow_good, shadow_coin, shadow_favo)
        xgnb.wait(5)

class Test_Shadow_02(Scene):
    CONFIG = {
        'camera_config': {
            'background_color': WHITE,
        },
    }
    def construct(xgnb):
        num=8
        colors = color_gradient([RED, PINK, BLUE, GREEN, YELLOW, ORANGE], num)
        s = 0.6
        circles = VGroup(*[Circle(radius=s * (i+0.6)) for i in range(num * 2)])
        shadows_out = VGroup(*[Shadow_2d(circles[i*2], blur_width=0.5 * s, layer_num=50, show_basic_shape=False, shadow_out=True) for i in range(num)], plot_depth=1)
        shadows_in = VGroup(*[Shadow_2d(circles[i*2+1], blur_width=0.5 * s, layer_num=50, show_basic_shape=False, shadow_out=False) for i in range(num-1)], plot_depth=1)
        annulus_group = VGroup(*[Annulus(inner_radius=s * (2 * i - 1 + 0.6), outer_radius=s * (2 * i + 0.6), fill_color=colors[i], fill_opacity=1, stroke_width=0, plot_depth=-5) for i in range(1, num)])
        circle = Circle(radius=0.6 * s, stroke_width=0, fill_color=colors[0], fill_opacity=1, plot_depth=1)

        xgnb.add(shadows_out, shadows_in, annulus_group, circle)
        xgnb.wait(5)

## test TransformMobject

class Test_TransformMobject(Scene):

    def construct(self):

        hexagon = RegularPolygon(6).set_width(4).set_stroke(width=1.2)
        hexagons = TransformMobject(hexagon, rotate_angle=-TAU, stroke_colors=WHITE, shift_vect=ORIGIN, num=100, rate_func=linear, scale_type=0)

        tri = RegularPolygon(3).set_width(2.).set_stroke(width=1.2).next_to(hexagon, UP, buff=0)
        tris = TransformMobject(tri, rotate_angle=2*PI/3, stroke_colors=WHITE, shift_vect=ORIGIN, num=40, rate_func=linear, scale_type=0)

        tris_group = VGroup(*[tris.copy().rotate(PI/3 * i, about_point=ORIGIN) for i in range(6)])

        p_list = [complex_to_R3((2 if i%2==0 else 2*np.sqrt(3)) * np.exp(TAU/12 * 1j * i)) for i in range(12)]
        out_line = Polygon(*p_list, stroke_width=6, color=WHITE)

        self.add(hexagons, tris_group, out_line)
        self.wait()


class Test_TransformMobject_02(Scene):

    def construct(self):

        square = Square().set_width(16).set_stroke(width=1.).round_corners(1.25)
        squares = TransformMobject(square, rotate_angle=TAU * 1.5, stroke_colors=WHITE, num=54)
        for i in range(len(squares)):
            if i%2 == 0:
                squares[i].set_fill(WHITE, 1).set_plot_depth(i)
            else:
                squares[i].set_fill(BLACK, 1).set_plot_depth(i)
        self.add(squares)
        self.wait()





from manimlib import *
from my_3D_mobject import *
class SpecialThreeDScene:
    pass

class Test_boxes(SpecialThreeDScene):
    CONFIG = {
        "default_angled_camera_position": {
            "phi": 70 * DEGREES,
            "theta": -45 * DEGREES,
            "distance": 50,
            },
        }
    def construct(self):

        self.set_camera_to_default_position()

        boxes = MyBoxes(fill_color=GREY_A, resolution=(24, 24), bottom_size=(0.3, 0.3), gap=0.1)
        # func_01 = lambda x, y: np.sin(x ** 2 / 2.1 + y ** 2 / 2.1 + 5 * DEGREES) * 1.8 + 2.
        R = lambda x, y: np.sqrt(x ** 2 * 4 + y ** 2 * 4) + 1e-8
        func_02 = lambda u, v: 8 * np.sin(R(u, v))/R(u, v) * 0.45 - 0.2
        boxes.update_top_by_func(func_02)
        boxes.update_color_by_func(func_02)

        mask = np.zeros((24, 24))
        for i in range(24):
            for j in range(24):
                if np.sqrt((i - 11.5) ** 2 + (j - 11.5) ** 2) > 9:
                    mask[i, j] = 1

        boxes.set_mask_array(mask)
        boxes.apply_mask()

        self.add(boxes)
        self.wait(2)

import matplotlib.pyplot as plt

def solve_PDE_onestep(array, dx=1, dy=1, kx=1e-1, ky=1e-1, Q=None, s=1.):
    m, n = len(array) + 2, len(array[0]) + 2
    T = np.zeros((m, n))
    T[1:-1, 1:-1] = array
    if type(Q) != np.ndarray:
        Q = np.zeros((m, n))
    # T_new = (kx * T[1:-1, 2:n+1] * dy ** 2 + kx * T[1:-1, 0:-2] * dy ** 2 +
    #          ky * T[2:n+1, 1:-1] * dx ** 2 + ky * T[0:-2, 1:-1] * dx ** 2 -
    #          (kx * dy ** 2 + ky * dx ** 2) * T[1:-1, 1:-1] +
    #          dx ** 2 * dy ** 2 * Q[1:-1, 1:-1]) / (kx * dy ** 2 + ky * dx ** 2)
    T_new = (kx * T[1:-1, 2:n+1] * dy ** 2 + kx * T[1:-1, 0:-2] * dy ** 2 +
             ky * T[2:n+1, 1:-1] * dx ** 2 + ky * T[0:-2, 1:-1] * dx ** 2 +
             dx ** 2 * dy ** 2 * Q[1:-1, 1:-1]) / (kx * dy ** 2 + ky * dx ** 2) / 2
    err = abs((T_new - array) ** 2).sum()/ abs(array).sum()
    # print(T_new)
    print(err)
    return T_new * s + array * (1 - s)

class Boxes_by_image(SpecialThreeDScene):

    CONFIG = {
        "default_angled_camera_position": {
            "phi": 45 * DEGREES,
            "theta": -0 * DEGREES,
            "distance": 50,
            },
        }
    def construct(self):

        self.set_camera_to_default_position()
        im = plt.imread(r'E:\GitHub\manim\my_manim_projects\my_projects\resource\png_files\china_test_60_45.bmp')
        Z = 1 - im[:, :, 0]/225 + 1e-6
        print(Z)
        Z = (Z - Z.min()) * 4
        Z = Z.T
        print(len(Z), len(Z[0]))
        print(Z.min(), Z.max())

        # print(Z)
        boxes = MyBoxes(fill_color=average_color(BLUE_D, GREY_A), bottom_size=(0.18, 0.18), resolution=(60, 45))
        boxes.colors = color_gradient([BLUE_D, YELLOW, ORANGE, RED, RED_D], 110)

        # for i in range(3):
        #     Z = solve_PDE_onestep(Z)

        boxes.update_top_and_bottom_by_2darray(Z, np.zeros((60, 45)))
        boxes.update_color_by_2darray(Z)

        for box in boxes:
            print(box.box_height)
            if box.box_height < 1e-4:
                box.set_fill(opacity=0)

        self.add(boxes)
        self.wait(4)

class Boxes_waves(SpecialThreeDScene):

    CONFIG = {
        "default_angled_camera_position": {
            "phi": 70 * DEGREES,
            "theta": -45 * DEGREES,
            "distance": 50,
            },
        }
    def construct(self):

        self.set_camera_to_default_position()
        axes = self.get_axes()
        boxes = MyBoxes(fill_color=GRAY, resolution=(20, 20), bottom_size=(0.25, 0.25), gap=0.05)
        self.var_phi = 0
        func_01 = lambda x, y: np.sin(x ** 2 / 2.4 + y ** 2 / 2.4 + self.var_phi) * 1.
        func_02 = lambda x, y: np.sin(x ** 2 / 2.4 + y ** 2 / 2.4 + self.var_phi) * 1. - 0.25

        boxes.update_top_and_bottom_by_func(func_01, func_02)
        boxes.update_color_by_func(func_01)
        def update_boxes(b, dt):
            b.update_top_and_bottom_by_func(func_01, func_02)
            b.update_color_by_func(func_01)
            self.var_phi += 1 * DEGREES
        self.add(boxes)
        boxes.add_updater(update_boxes)
        # self.wait(2)
        # boxes.remove_updater(update_boxes)
        self.wait(12)

# test Cube_array

class Test_cube_array(ThreeDScene):

    CONFIG = {
        'camera_init': {
            'phi': 52.5 * DEGREES,
            'gamma': 0,
            'theta': -45 * DEGREES,
        },
        # 'camera_config': {
        #     'should_apply_shading': False,
        # },
    }

    def construct(self):

        self.set_camera_orientation(**self.camera_init)

        cubes = Cube_array(fill_color=BLUE, fill_opacity=0.6, resolution=(3,3,3), cube_size=1)
        self.add(cubes)
        self.wait()
        self.play(cubes.outer_faces.shift, DL * 1.5, cubes.inner_faces.shift, UR * 2, run_time=2)

        self.wait(3)

class Rubik_Cube_test(ThreeDScene):

    CONFIG = {
        'camera_init': {
            'phi': 52.5 * DEGREES,
            'gamma': 0,
            'theta': -45 * DEGREES,
        },
    }

    def construct(self):

        self.set_camera_orientation(**self.camera_init)

        rc = Rubik_Cube()

        self.add(rc)
        self.wait()

        t = 1.6
        self.play(Rotating(rc.get_layer(1, 1), radians=PI/2, axis=RIGHT, run_time=t))

        self.play(Rotating(rc.get_layer(1, 3), radians=-PI/2, axis=OUT, run_time=t))

        self.play(Rotating(rc.get_layer([1,3], 2), radians=PI, axis=UP, run_time=t), Rotating(rc.get_layer(2, 2), radians=PI*0, axis=UP, run_time=t))

        self.play(Rotating(rc.get_layer(2, 1), radians=PI/2, axis=RIGHT, run_time=t), Rotating(rc.get_layer([1,3], 1), radians=PI/2 * 0, axis=RIGHT, run_time=t))

        self.play(Rotating(rc.get_layer(1, 3), radians=-PI/2, axis=OUT, run_time=t))

        self.play(Rotating(rc.get_layer([1,2], 2), radians=-PI/2, axis=UP, run_time=t), Rotating(rc.get_layer(3, 2), radians=PI*0, axis=UP, run_time=t))

        self.play(Rotating(rc.get_layer([3,2], 1), radians=PI/2, axis=RIGHT, run_time=t), Rotating(rc.get_layer(1, 1), radians=PI/2 * 0, axis=RIGHT, run_time=t))

        self.wait(2)

class Play_rubic_order3(Rubik_Scene):

    CONFIG = {
        'order': 3,
    }

    def construct(self):

        self.wait()
        self.rotate_rubik_anim(1, 3, 1)       # U'
        self.rotate_rubik_anim(1, 1, -1)      # R
        self.rotate_rubik_anim(0, 2, 1)       # y
        self.rotate_rubik_anim(1, 3, 2)       # U'2
        self.rotate_rubik_anim(2, 1, -2)      # M'2
        self.rotate_rubik_anim([1,2], 3, -2)  # u2
        self.rotate_rubik_anim(1, 1, -1)      # L'
        self.rotate_rubik_anim([2,3], 2, 1)   # b
        self.rotate_rubik_anim(1, 3, -1)      # U
        self.wait()


class Play_rubic_order5(Rubik_Scene):

    CONFIG = {
        'order': 5,
    }

    def construct(self):

        self.wait()
        self.rotate_rubik_anim(1, 3, 1)
        self.rotate_rubik_anim(1, 1, -1)
        self.rotate_rubik_anim(0, 2, 1)
        self.rotate_rubik_anim(1, 3, 2)
        self.rotate_rubik_anim(2, 1, -2)
        self.rotate_rubik_anim([1,2], 3, -2)
        self.rotate_rubik_anim(1, 1, -1)
        self.rotate_rubik_anim([2,3], 2, 1)
        self.rotate_rubik_anim(1, 3, -1)
        self.wait()



import random
import time
import shutil
# from manimlib.utils.config_ops import digest_config
# from manimlib.scene.scene_file_writer import SceneFileWriter
from manimlib import Scene, Point, Camera, ShowCreation, Write, Color, VGroup
from manimlib.extract_scene import get_scene_config
import manimlib.config
from manimlib.config import Size
from .tools import ppath
from .plot import Plot
from .onlinetex import tex_to_svg_file_online
import manimlib.mobject.svg.tex_mobject

__all__ = ["EagerModeScene", "JupyterModeScene", "Size", "SceneArgs"]


class SceneArgs:
    # write_file = False
    # file_name = None
    # skip_animations = False  # "Save the last frame"
    color = None  # Background color"
    full_screen = False
    gif = False
    resolution = '1920x1080'

    # Render to a movie file with an alpha channel,
    # if transparent is True, .mov file will be generated.
    transparent = False
    save_pngs = False  # Save each frame as a png
    hd = False
    uhd = False
    quiet = True
    open = False  # Automatically open the saved file once its done
    finder = False  # Show the output file in finder
    frame_rate = None
    video_dir = None  # directory to write video
    start_at_animation_number = None
    use_online_tex = False


class EagerModeScene(Scene):
    def __init__(
        self,
        write_file=False,
        file_name=None,
        screen_size=Size.big,
        scene_name='EagerModeScene',
        CONFIG=None,
    ):
        self.CONFIG = CONFIG
        args = manimlib.config.parse_cli()
        args_dict = vars(args)
        args_dict['file'] = None
        args_dict['scene_names'] = scene_name
        args_dict['screen_size'] = screen_size
        for key, value in SceneArgs.__dict__.items():
            args_dict[key] = value

        if write_file is True or SceneArgs.gif is True:
            args_dict['write_file'] = True
            args_dict['file_name'] = file_name
            if SceneArgs.gif is True:
                args_dict["transparent"] = False

        if SceneArgs.use_online_tex:
            print("Use online latex compiler")
            manimlib.mobject.svg.tex_mobject.tex_to_svg_file = tex_to_svg_file_online

        self.config = manimlib.config.get_configuration(args)
        self.scene_config = get_scene_config(self.config)

        super().__init__(**self.scene_config)

        self.virtual_animation_start_time = 0
        self.real_animation_start_time = time.time()
        self.file_writer.begin()

        self.setup()
        self.plt = Plot()

        self.episodes = []
        self.current_episode = 1
        self.current_animation = 0
        self.loop_start_animation = None
        self.pause_start_animation = 0

    def play(self, *args, run_time=1, **kwargs):
        """TODO:"""
        super().play(*args, run_time=run_time, **kwargs)
        self.current_animation += 1

    def hold_on(self):
        """ Equal to self.tear_down(). """
        self.stop_skipping()
        self.file_writer.finish()
        if self.window and self.linger_after_completion:
            self.interact()

    def get_config(self):
        return self.config

    def save_default_config(self):
        """Save the default config file to current directory."""
        shutil.copy(ppath("custom_config.yml"), 'custom_config.yml')

    def get_scene_config(self):
        return self.scene_config

    def save_start(self, file_name):
        """TODO"""
        pass

    def save_end(self):
        # self.file_writer.finish()
        pass

    def embed(self):
        super().embed()

    def plot(self,
             x,
             y,
             color=None,
             width=2,
             axes_ratio=0.62,
             scale_ratio=None,
             num_decimal_places=(1, 1),
             show_axes=True,
             include_tip=True,
             x_label='x',
             y_label='y'):

        """
        params
        ------

        scale_ratio: Scale ratio of coordinate axis. i.e. y / x .
        num_decimal_places: Number of significant digits of coordinate_labels.
        """
        self.plt.plot(x, y, color, width, axes_ratio, scale_ratio, show_axes, include_tip, num_decimal_places,
                      x_label, y_label)

    def plot3d(self, x, y, z, width=2, axes_ratio=0.62, show_axes=True):
        """TODO"""
        pass

    def get_plot_mobj(self):
        self.plt.gen_axes_lines()
        return self.plt.get_axes_lines()

    def get_plot_axes(self):
        return self.plt.get_axes()

    def show_plot(self, play=True):
        axes_lines_dict = self.get_plot_mobj()

        random.seed(time.time())
        if play:
            def play_func(Func):
                if axes_lines_dict['axes']:
                    self.play(ShowCreation(VGroup(*axes_lines_dict["axes"])),
                              run_time=1)
                self.play(Func(VGroup(*axes_lines_dict["line"])), run_time=1)

            if random.random() > 0.5:
                play_func(Write)
            else:
                play_func(ShowCreation)
        else:
            self.add(VGroup(*axes_lines_dict["line"],
                            *axes_lines_dict["axes"]))

        self.plt = Plot()


class JupyterModeScene(EagerModeScene):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def hold_on(self):
        self.file_writer.finish()

    def embed(self):
        """We don't need it in jupyter lab/notebook."""
        pass

    def quit(self):
        """Please use exit() or quit() in jupyter cell."""
        pass



import numpy as np
from manimlib import *


class Quaternion:
    def __init__(self, x=None, y=0, z=0, w=1):
        """Quaternion style [x, y, z, w]"""
        if issubclass(type(x), (np.ndarray, list, tuple)):
            self._x = x[0]
            self._y = x[1]
            self._z = x[2]
            self._w = x[3]
        else:
            if x is None:
                x = 1
            self._x = x
            self._y = y
            self._z = z
            self._w = w

        self._vec = np.array([self._x, self._y, self._z])
        self._q = np.array([*self._vec, self._w])

    def _set_q(self):
        self._vec = np.array([self._x, self._y, self._z])
        self._q = np.array([*self._vec, self._w])

    def to_array(self):
        return self._q

    def normalise(self):
        L = np.linalg.norm(self._vec)
        # self._q /= L
        self._x /= L
        self._y /= L
        self._z /= L
        self._w /= L
        self._set_q()

    def slerp(self):
        """TODO"""
        pass

    def multi(self, *quats):
        q = self
        for qi in quats:
            q = Quaternion.multiply_quat_2(q, qi)
        self._vec = q._vec
        self._q = q._q
        # self._set_q()
        return q

    @staticmethod
    def multiply_quat(q1, q2):
        """reference http://www.euclideanspace.com/maths/algebra/realNormedAlgebra/quaternions/code/index.htm"""
        x = q1.x * q2.w + q1.y * q2.z - q1.z * q2.y + q1.w * q2.x
        y = -q1.x * q2.z + q1.y * q2.w + q1.z * q2.x + q1.w * q2.y
        z = q1.x * q2.y - q1.y * q2.x + q1.z * q2.w + q1.w * q2.z
        w = -q1.x * q2.x - q1.y * q2.y - q1.z * q2.z + q1.w * q2.w
        new_q = object.__new__(Quaternion)
        new_q.__init__(x, y, z, w)
        return new_q

    @staticmethod
    def multiply_quat_2(q1, q2):
        """Graßmann Product"""
        v1 = q1._vec
        v2 = q2._vec
        w1 = q1._w
        w2 = q2._w
        vec = w1 * v2 + w2 * v1 + np.cross(v1, v2)
        w = w1 * w2 - v1.dot(v2)

        new_q = object.__new__(Quaternion)
        new_q.__init__([*vec, w])
        return new_q

    def __new__(cls, *args, **kwargs):
        return object.__new__(cls)

    def copy(self):
        obj = object.__new__(Quaternion)
        obj.__init__(*self._q)
        return obj

    def set_x(self, value):
        self._x = value
        self._set_q()

    def set_y(self, value):
        self._y = value
        self._set_q()

    def set_z(self, value):
        self._z = value
        self._set_q()

    def set_w(self, value):
        self._w = value
        self._set_q()

    def set_from_euler(self):
        """TODO"""
        pass

    def set_from_axis_angle(self, axis: np.ndarray, angle):
        axis = normalize(np.array(axis))
        half_angle = angle / 2
        s = np.sin(half_angle)

        self._x = axis[0] * s
        self._y = axis[1] * s
        self._z = axis[2] * s
        self._w = np.cos(half_angle)
        self._set_q()

        return self

    def conjugate(self, in_place=True):
        if in_place:
            self._vec *= -1
            self._set_q()
            return self
        else:
            q = self.copy()
            q._vec *= -1
            q._set_q()
            return q

    def invert(self):
        return self.conjugate()

    def dot(self, v):
        return self._q.dot(v)

    def __str__(self):
        return self._q.__str__()

    @property
    def x(self):
        return self._vec[0]

    @property
    def y(self):
        return self._vec[1]

    @property
    def z(self):
        return self._vec[2]

    @property
    def w(self):
        return self._w


if __name__ == "__main__":
    axis = np.array([1, 1, 1])
    q1 = Quaternion().set_from_axis_angle(axis, 20 * DEGREES)
    q2 = Quaternion().set_from_axis_angle(axis, 30 * DEGREES)
    print(Quaternion.multiply_quat(q1, q2))
    print(Quaternion.multiply_quat_2(q1, q2))



from manim_express import EagerModeScene
from manimlib import *
# from manimlib.utils.config_ops import digest_config


class CustomSurface:
    def __init__(self, axes=None, resolution=(101, 51)):
        if axes is None:
            axes = ThreeDAxes()
            # axes.add_coordinate_labels(
            #     font_size=15,
            # )
        self.axes = axes
        self.resolution = resolution

    def set_default_kwargs(self, kwargs: dict):
        kwargs.setdefault('color', BLUE_E)
        kwargs.setdefault('resolution', self.resolution)

    def get_axes(self):
        return self.axes

    def cylinder(self, r=1, h=3, **kwargs):
        """https://www2.math.uconn.edu/~stein/math210/slides/math210-09notes.pdf"""
        self.set_default_kwargs(kwargs)
        return ParametricSurface(lambda u, v: self.axes.c2p(*np.array(
            [r * np.cos(v), r * np.sin(v), u])),
                                 u_range=[-EPSILON, EPSILON + h],
                                 v_range=[0, TAU],
                                 color=kwargs['color'],
                                 resolution=self.resolution)

    def paraboloid(self, **kwargs):
        self.set_default_kwargs(kwargs)
        return ParametricSurface(lambda u, v: self.axes.c2p(*np.array(
            [np.cos(v) * u, np.sin(v) * u, u ** 2])),
                                 u_range=[-EPSILON - 2, EPSILON + 2],
                                 v_range=[-2, 2],
                                 color=kwargs['color'],
                                 resolution=self.resolution)

    def para_hyp(self, **kwargs):
        self.set_default_kwargs(kwargs)
        return ParametricSurface(
            lambda u, v: self.axes.c2p(*np.array([u, v, u ** 2 - v ** 2])),
            v_range=[-2, 2],
            u_range=[-2, 2],
            color=kwargs['color'],
            resolution=self.resolution)

    def cone(self, **kwargs):
        self.set_default_kwargs(kwargs)
        return ParametricSurface(lambda u, v: self.axes.c2p(*np.array(
            [u * np.cos(v),
             u * np.sin(v),
             u])),
                                 v_range=[0, TAU],
                                 u_range=[-EPSILON - 2, EPSILON + 2],
                                 color=kwargs['color'],
                                 resolution=self.resolution)

    def hip_one_side(self, **kwargs):
        self.set_default_kwargs(kwargs)
        return ParametricSurface(lambda u, v: self.axes.c2p(*np.array(
            [np.cosh(u) * np.cos(v),
             np.cosh(u) * np.sin(v),
             np.sinh(u)])),
                                 v_range=[0, TAU],
                                 u_range=[-EPSILON - 2, EPSILON + 2],
                                 color=kwargs['color'],
                                 resolution=self.resolution)

    def ellipsoid(self, a, b, c, **kwargs):
        """https://zh.wikipedia.org/wiki/%E6%A4%AD%E7%90%83"""
        return ParametricSurface(lambda u, v: self.axes.c2p(*np.array([
            a * np.cos(u) * np.cos(v),
            b * np.cos(u) * np.sin(v),
            c * np.sin(u)
        ])),
                                 v_range=[0, TAU],
                                 u_range=[-EPSILON - PI / 2, EPSILON + PI / 2],
                                 resolution=self.resolution)

    def sphere(self, r, origin=(0, 0, 0), **kwargs):
        """https://zh.wikipedia.org/wiki/%E7%90%83%E9%9D%A2"""

        self.set_default_kwargs(kwargs)
        return ParametricSurface(
            lambda u, v: self.axes.c2p(*np.array([
                r * np.sin(u) * np.cos(v) + origin[0],
                r * np.sin(u) * np.sin(v) + origin[1],
                r * np.cos(u) + origin[2]
                # r * np.sin(v) * np.cos(u) + origin[0],
                # r * np.sin(v) * np.sin(u) + origin[1],
                # r * np.cos(v) + origin[2]
            ])),
            u_range=[-EPSILON, PI + EPSILON],
            v_range=[0, TAU],
            # v_range=[-EPSILON, PI + EPSILON], u_range=[0, TAU],

            color=kwargs['color'],
            resolution=self.resolution)

    def circle(self, r, origin=(0, 0, 0), **kwargs):
        self.set_default_kwargs(kwargs)
        return ParametricSurface(
            lambda u, v: self.axes.c2p(*np.array([
                r * np.sin(u) * np.cos(v) + origin[0],
                r * np.sin(u) * np.sin(v) + origin[1],
                origin[2]
            ])),
            u_range=[-EPSILON, PI + EPSILON],
            v_range=[0, TAU],
            # v_range=[-EPSILON, PI + EPSILON], u_range=[0, TAU],
            color=kwargs['color'],
            resolution=self.resolution)

    def square(self, w, h, vertex1=(0, 0, 0), color=BLUE_E):
        return ParametricSurface(
            lambda u, v: self.axes.c2p(*np.array([
                u + vertex1[0],
                v + vertex1[1],
                vertex1[2]

            ])),
            u_range=[0, w],
            v_range=[0, h],
            # v_range=[-EPSILON, PI + EPSILON], u_range=[0, TAU],
            color=color,
            resolution=self.resolution)

    def torus(self, R=3, r=1, **kwargs):
        self.set_default_kwargs(kwargs)
        return ParametricSurface(
            lambda u, v: self.axes.c2p(*np.array([
                (R+r*np.cos(v))*np.cos(u),
                (R+r*np.cos(v))*np.sin(u),
                r*np.sin(v)
            ])),
            u_range=[0, TAU],
            v_range=[0, TAU],
            color=kwargs['color'],
            resolution=kwargs['resolution']
        )





if __name__ == "__main__":
    s = EagerModeScene()
    # axes = ThreeDAxes()
    surface = CustomSurface()
    axes = surface.get_axes()
    s.add(axes)

    frame = s.camera.frame
    frame.set_euler_angles(
        # theta = -10 * DEGREES
        phi=75 * DEGREES)
    sphere = surface.sphere(2, [3, 0, 0])
    sphere.set_color_by_gradient((BLUE_E, RED_E))

    cylinder = surface.cylinder(2).move_to(axes.c2p(-2, 0, 0))
    cylinder.set_color_by_gradient((YELLOW_D, BLUE_D))

    s.play(ShowCreation(sphere))
    s.play(ShowCreation(cylinder))
    s.hold_on()



from manimlib.constants import *
from manimlib.mobject.types.vectorized_mobject import VMobject, VGroup
from manimlib.mobject.geometry import Arc, Line, Dot, Polygon, Sector, Circle
from manimlib.utils.color import color_gradient
from manimlib.mobject.number_line import DecimalNumber
from manimlib.mobject.svg.tex_mobject import Tex
from manimlib.mobject.svg.text_mobject import Text
from manimlib.utils.rate_functions import linear, smooth
from manimlib.utils.space_ops import *

class Arcs(VGroup):

    CONFIG = {
        'colors': [RED, YELLOW, BLUE, PINK],
        'radius': 1,
        'start_angle':0,
        'angle_list': [30 * DEGREES, 60 * DEGREES, 90 * DEGREES],
        'stroke_width': 40,
    }

    def __init__(self, **kwargs):

        VMobject.__init__(self, **kwargs)
        self.create_arcs()

    def create_arcs(self, **kwargs):
        angle = self.start_angle
        colors = color_gradient(self.colors, len(self.angle_list))
        for i in range(len(self.angle_list)):
            self.add(Arc(radius=self.radius, start_angle=angle, angle=self.angle_list[i], color=colors[i], stroke_width=self.stroke_width, **kwargs))
            angle += self.angle_list[i]

class Angle(VGroup):

    CONFIG = {
        'radius': 1,
        'color': RED,
        'opacity': 0.4,
        'stroke_width': 10,
        # 'below_180': True,
    }
    def __init__(self, A, O, B, **kwargs):

        VMobject.__init__(self, **kwargs)
        OA, OB = A-O, B-O
        theta = np.angle(complex(*OA[:2])/complex(*OB[:2])) # angle of OB to OA

        self.add(Arc(start_angle=Line(O, B).get_angle(), angle=theta, radius=self.radius/2,
                     stroke_width=100 * self.radius, color=self.color).set_stroke(opacity=self.opacity).move_arc_center_to(O))
        self.add(Arc(start_angle=Line(O, B).get_angle(), angle=theta, radius=self.radius,
                     stroke_width=self.stroke_width, color=self.color).move_arc_center_to(O))

class Tracked_Point(VGroup):
    CONFIG = {
        'size': 0.1,
        'point_color': BLUE,
        'num_decimal_places': 2,
        'coordinates_scale': 0.8,
        'coordinates_color': GREEN,
        'coordinates_direction': DOWN * 0.25,
        'bracket_color': WHITE,
    }
    def __init__(self, init_loc=ORIGIN, **kwargs):

        VGroup.__init__(self, **kwargs)
        self.point = Dot(init_loc, color=self.point_color).set_height(self.size)
        self.value_x = DecimalNumber(0, color=self.coordinates_color, num_decimal_places=self.num_decimal_places).scale(self.coordinates_scale)
        self.value_y = DecimalNumber(0, color=self.coordinates_color, num_decimal_places=self.num_decimal_places).scale(self.coordinates_scale)
        text = Tex('(', ',', ')').scale(self.coordinates_scale)
        self.coordinates_text = VGroup(text[0], self.value_x, text[1], self.value_y, text[2])
        self.coordinates_text.add_updater(self.update_coordinates_text)
        self.add(self.point)

    def update_coordinates_text(self, coords):
        for i in range(1, len(coords)):
            coords[i].next_to(coords[i-1], RIGHT * 0.5)
        coords[2].align_to(coords[1], DOWN)
        pos = self.point.get_center()
        x, y = self.mapping_func(pos[0], pos[1])
        coords[1].set_value(x)
        coords[3].set_value(y)
        coords.next_to(self.point, self.coordinates_direction)

    def mapping_func(self, x, y):
        return x, y

class Dashed_Circle(VGroup):

    CONFIG = {
        'arc_ratio': 0.6,
        'arc_num': 36,
        'arc_config':{
            'color': WHITE,
            'stroke_width': 2.5,
        },
    }

    def __init__(self, radius=1, center=ORIGIN, **kwargs):
        VGroup.__init__(self, **kwargs)
        theta = TAU/self.arc_num
        for i in range(self.arc_num):
            arc_i = Arc(radius=radius, angle=theta * self.arc_ratio, **self.arc_config)
            arc_i.rotate(theta * i, about_point=ORIGIN)
            self.add(arc_i)
        self.move_to(center)

class Right_angle(VGroup):
    CONFIG = {
        'size': 0.25,
        'stroke_color': WHITE,
        'stroke_width': 3.2,
        'fill_color': BLUE,
        'fill_opacity': 0.5,
        'on_the_right': True,
    }
    def __init__(self, corner=ORIGIN, angle=0, **kwargs):
        VGroup.__init__(self, **kwargs)
        self.corner = ORIGIN
        self.angle = 0
        r = UR if self.on_the_right else UL
        self.add(Polygon(ORIGIN, RIGHT * self.size * r, UR * self.size * r, UP * self.size * r, stroke_width=0,
                         fill_color=self.fill_color, fill_opacity=self.fill_opacity),
                 Line(RIGHT * self.size * r, UR * self.size * r + UP * self.stroke_width/100/2 * 0.8, stroke_width=self.stroke_width, stroke_color=self.stroke_color),
                 Line(UR * self.size * r + RIGHT * self.stroke_width/100/2 * r * 0.8, UP * self.size * r, stroke_width=self.stroke_width, stroke_color=self.stroke_color),
                 )
        self.move_corner_to(corner)
        self.change_angle_to(angle)

    def move_corner_to(self, new_corner):
        self.shift(new_corner - self.corner)
        self.corner = new_corner
        return self

    def change_angle_to(self, new_angle):
        self.rotate(new_angle - self.angle, about_point=self.corner)
        self.angle = new_angle
        return self

class Trail(VGroup):

    CONFIG = {
        'max_width': 5,
        'nums': 500,
        'trail_color': BLUE_B,
        # 'rate_func': linear,
        'rate_func': lambda t: t ** 1.25,
    }

    def __init__(self, mob, **kwargs):
        VGroup.__init__(self, **kwargs)
        self.add(mob)
        self.trail = VGroup()
        self.path_xyz = []
        self.add(self.trail)
        self.pos_old = self[0].get_center()
        if type(self.trail_color) != str:
            self.colors = color_gradient(self.trail_color, self.nums)

    # def update_trail(self, trail):
    #     err=1e-5
    #     pos_new = self[0].get_center()
    #     pos_old = self.pos_old
    #     self.pos_old = pos_new
    #     # if np.sqrt(sum((pos_new - pos_old) ** 2))>err:
    #     if sum(abs(pos_new - pos_old))>err:
    #         trail.add(Line(pos_old, pos_new, color=self.trail_color, plot_depth=0))
    #
    #     if len(trail) > self.nums:
    #         trail.remove(trail[0])
    #         # for k in range(self.nums):
    #         #     trail[k].set_stroke(width=self.max_width * self.rate_func(k/self.nums),
    #         #                         opacity=self.rate_func(k/self.nums))
    #         for l in trail:
    #             k = trail.submobjects.index(l)
    #             l.set_stroke(width=self.max_width * self.rate_func(k/self.nums),
    #                          opacity=self.rate_func(k/self.nums))
    #
    #     if len(trail) <= self.nums and len(trail) > 0:
    #         # for k in range(len(trail)):
    #         #     trail[k].set_stroke(width=self.max_width * self.rate_func(k/len(trail)),
    #         #                         opacity=self.rate_func(k/len(trail)))
    #         for l in trail:
    #             k = trail.submobjects.index(l)
    #             l.set_stroke(width=self.max_width * self.rate_func(k/len(trail)),
    #                          opacity=self.rate_func(k/len(trail)))

    def get_path_xyz(self, err=1e-6):
        pos_new = self[0].get_center()
        pos_old = self.pos_old
        if sum(abs(pos_new - pos_old))>err:
            self.path_xyz.append(pos_new)
        self.pos_old = pos_new
        while len(self.path_xyz) > self.nums:
            self.path_xyz.remove(self.path_xyz[0])

    def create_path(self):
        path = VGroup()
        self.get_path_xyz()
        if len(self.path_xyz) > 1:
            for i in range(len(self.path_xyz)-1):
                if type(self.trail_color) == str:
                    path.add(Line(self.path_xyz[i], self.path_xyz[i+1], stroke_color=self.trail_color,
                                  stroke_opacity=self.rate_func(i/len(self.path_xyz)), plot_depth=self.rate_func(2-i/len(self.path_xyz)),
                                  stroke_width=self.max_width * self.rate_func(i/len(self.path_xyz))))
                else:
                    path.add(Line(self.path_xyz[i], self.path_xyz[i+1], stroke_color=self.colors[i],
                                  stroke_opacity=self.rate_func(i/len(self.path_xyz)), plot_depth=self.rate_func(2-i/len(self.path_xyz)),
                                  stroke_width=self.max_width * self.rate_func(i/len(self.path_xyz))))
                # print('i = %d' % i)
                # # print(self.path_xyz)
                # print(self.color)
                # print(self.rate_func(i/len(self.path_xyz)))
                # print(self.max_width*self.rate_func(i/len(self.path_xyz)))
        return path

    def update_path(self, trail):
        trail.become(self.create_path())

    def start_trace(self):
        # self.trail.add_updater(self.update_trail)
        self.trail.add_updater(self.update_path)

    def stop_trace(self):
        self.trial.remove_updater(self.update_path)

    def decrease_trail_num(self, trail, dt):
        if self.nums > max(self.min_num, 2):
            if self.nums <= 2:
                trail.become(VGroup())
            else:
                self.nums -= self.rate
                if self.nums < 2:
                    self.nums = 2
                trail.become(self.create_path())

    def retrieve_trail(self, rate=2, min_num=0):
        # self.stop_trace()
        self.nums = len(self.trail)
        self.min_num = min_num
        self.rate = rate
        self.trail.add_updater(self.decrease_trail_num)

class Sun(VGroup):
    CONFIG = {
        'colors': [RED_B, ORANGE, WHITE],
        # 'opacity_func': lambda t: 1.1 - t ** 0.24 if t < 0.1 else 1 - 0.95 * t ** 0.18 - 0.05 * t ** 0.05,
        # 'opacity_func': lambda t: 1000 * (1 - t ** 0.00012) if t < 0.1 else 0.75 * (1 - t ** 0.21),
        # 'opacity_func': lambda t: 1250 * (1 - abs(t-0.006) ** 0.0001) if t < 0.12 else 0.72 * (1 - t ** 0.2),
        'opacity_func': lambda t: 1500 * (1 - abs(t-0.009) ** 0.0001),
        'radius': 4,
        'layer_num': 80,
        # 'rate_func': smooth,
        'rate_func': lambda t: t ** 2,
    }
    def __init__(self, **kwargs):

        VGroup.__init__(self, **kwargs)
        self.color_list = color_gradient(self.colors, self.layer_num)
        self.add(Dot(color=color_gradient(self.colors[0], WHITE), plot_depth=4).set_height(0.015 * self.radius))
        for i in range(self.layer_num):
            # self.add(Arc(radius= self.radius/self.layer_num * (0.5 + i), angle=TAU, color=self.color_list[i],
            #              stroke_width=100 * self.radius/self.layer_num,
            #              stroke_opacity=self.opacity_func(i/self.layer_num), plot_depth=5))
            self.add(Arc(radius= self.radius * self.rate_func((0.5 + i)/self.layer_num), angle=TAU, color=self.color_list[i],
                         stroke_width=101 * (self.rate_func((i + 1)/self.layer_num) - self.rate_func(i/self.layer_num)) * self.radius,
                         stroke_opacity=self.opacity_func(self.rate_func(i/self.layer_num)), plot_depth=5))

class Three_Body(VGroup):

    CONFIG = {
        'mass': np.array([0.98, 1.025, 1]) * 1.2,
        'pos': np.array([[-3., -np.sqrt(3), 0], [0., 3 * np.sqrt(3) - 1, 0], [3, -np.sqrt(3), 0]]) * 0.75,
        'velocity': np.array([[1, -np.sqrt(3), 0], [-2, 0, 0], [1, np.sqrt(3), 0]]) * 0.8,
        'p_pos': np.array([2, -np.sqrt(3)+1, 0]) * 1.,
        'p_velocity':np.array([-1, -1.7, 0]) * 2.4,
        'plot_depth':5,
    }

    def __init__(self, *three_Mobject, **kwargs):

        VGroup.__init__(self, **kwargs)

        self.sun_01 = three_Mobject[0].move_to(self.pos[0])
        self.sun_02 = three_Mobject[1].move_to(self.pos[1])
        self.sun_03 = three_Mobject[2].move_to(self.pos[2])
        if len(three_Mobject) > 3:
            self.planet = three_Mobject[3].move_to(self.p_pos)
        self.add(self.sun_01, self.sun_02, self.sun_03)
        if len(three_Mobject) > 3:
            self.planet = three_Mobject[3].move_to(self.p_pos)
            self.add(self.planet)

    def get_force(self, x1, x2, m1, m2, G=1):
        # force of obj_01 to obj_02, this vector start from obj_02 and end in obj_01
        r = np.sqrt(sum((x1 - x2) ** 2))
        return G * m1 * m2 * (x1 - x2) / (r ** 3 + 2e-3)

    def update_xyz(self, G=1, delta_t =2.5e-3):

        m1, m2, m3 = self.mass[0], self.mass[1], self.mass[2]
        x1, x2, x3 = self.pos[0], self.pos[1], self.pos[2]
        v1, v2, v3 = self.velocity[0], self.velocity[1], self.velocity[2]
        f21, f31, f32 = self.get_force(x2, x1, m2, m1, G=G), self.get_force(x3, x1, m3, m1, G=G), self.get_force(x3, x2, m3, m2, G=G)
        a1, a2, a3 = (f21 + f31) / m1, (-f21 + f32) / m2, (-f32 - f31) / m3

        xp, vp = self.p_pos, self.p_velocity
        f1, f2, f3 = self.get_force(x1, xp, m1, 1, G=G), self.get_force(x2, xp, m2, 1, G=G), self.get_force(x3, xp, m3, 1, G=G)
        a = (f1 + f2 + f3) / 1.

        self.velocity[0] += a1 * delta_t
        self.velocity[1] += a2 * delta_t
        self.velocity[2] += a3 * delta_t
        self.p_velocity += a * delta_t

        self.pos[0] += v1 * delta_t
        self.pos[1] += v2 * delta_t
        self.pos[2] += v3 * delta_t
        self.p_pos += vp *delta_t

    def reset_velocity(self):
        v1, v2, v3 = self.velocity[0], self.velocity[1], self.velocity[2]
        m1, m2, m3 = self.mass[0], self.mass[1], self.mass[2]
        momentum = v1 * m1 + v2 * m2 + v3 * m3
        v = momentum/(m1 + m2 + m3)
        v1, v2, v3 = v1 - v, v2 - v, v3 - v
        print(v1, v2, v3)
        self.p_velocity -= v
        self.velocity = np.array([v1, v2, v3])

    def update_three_body(self, tb, dt):
        self.update_xyz(G=40)
        # avervage_pos = (self.pos[0] + self.pos[1] + self.pos[2]) / 3
        # tb[0].move_to(self.pos[0] - avervage_pos)
        # tb[1].move_to(self.pos[1] - avervage_pos)
        # tb[2].move_to(self.pos[2] - avervage_pos)
        # if len(tb)>3:
        #     tb[3].move_to(self.p_pos - avervage_pos)
        tb[0].move_to(self.pos[0])
        tb[1].move_to(self.pos[1])
        tb[2].move_to(self.pos[2])
        if len(tb)>3:
            tb[3].move_to(self.p_pos)

    def start_move(self):
        self.add_updater(self.update_three_body)

class MySectors(VGroup):
    CONFIG = {
        'stroke_width': 0,
        'fill_opacity': 1,
        'inner_radius': 1.6,
        # 'outer_radius': [],
        'gap': 0.025,
        'start_direction': UP,
        'values': [1,2,3],
        'labels': None,
        # 'data': {'labels': 1.23},
        'unit': None,
        # 'data_2d': None,
        'outer_radius_func': lambda t: t/10 + 0.32,
        'label_font': '思源黑体 Bold',
        'center': ORIGIN,
    }

    def __init__(self, **kwargs):
        VGroup.__init__(self, **kwargs)
        self.colors = color_gradient([ORANGE, RED, PINK, BLUE, GREEN, YELLOW], len(self.values))
        self.sectors, self.labels_group = VGroup(), VGroup()
        self.sectors = self.create_sectors()
        if not self.labels == None:
            self.labels_group = self.create_label()
        self.add(self.sectors, self.labels_group)


    def create_sectors(self):
        angle = TAU/len(self.values)
        colors = self.colors
        start_a = np.angle(complex(*self.start_direction[0:2]))

        for i in range(len(self.values)):
            r_i = self.inner_radius + self.outer_radius_func(self.values[i])
            sector_i = Sector(arc_center=self.center, inner_radius=self.inner_radius, outer_radius=r_i,
                              stroke_width=self.stroke_width, start_angle=start_a + i * angle,
                              angle=angle * (1 - self.gap), color=colors[i], fill_opacity=self.fill_opacity)
            self.sectors.add(sector_i)
        return self.sectors

    def create_label(self):
        for tex, value in zip(self.labels, self.values):
            i = self.labels.index(tex)
            r = self.inner_radius + self.outer_radius_func(self.values[i])
            size = TAU * r / len(self.values) * 0.2
            tex_i = Text(tex, font=self.label_font, color=WHITE, plot_depth=1).set_height(size)
            value_i = Text(str(value), font=self.label_font, color=WHITE, plot_depth=1).set_height(size).next_to(tex_i, DOWN * 0.64 * size)
            if not self.unit == None:
                unit_i = Text(self.unit, font=self.label_font, color=WHITE, plot_depth=1).set_height(size).next_to(value_i, RIGHT * 0.2 * size)
                VGroup(value_i, unit_i).next_to(tex_i, DOWN * 0.64 * size)
                label_i = VGroup(tex_i, value_i, unit_i)
            else:
                label_i = VGroup(tex_i, value_i)
            angle = TAU/len(self.values)
            start_a = np.angle(complex(*self.start_direction[0:2]))
            self.labels_group.add(label_i.shift(self.center + complex_to_R3((r-size * 1.2-r*0.05) * np.exp(1j * (start_a + (i + 0.5) * TAU/len(self.values))))))
        return self.labels_group

    def create_cicles(self, color=BLUE_A):

        circle_01 = Circle(radius=self.inner_radius, stroke_width=12, stroke_color=color, plot_depth=2.5)
        circle_02 = Circle(radius=self.inner_radius - 0.15, stroke_width=4, stroke_color=color, plot_depth=2.5)
        self.circles = VGroup(circle_01, circle_02).move_to(self.center)
        self.add(self.circles)
        return self.circles

    def create_circle_shadow(self, width=32, num=50, color=BLUE_A):
        self.shadow = VGroup(*[Circle(radius=self.inner_radius + (i+0.5) * width/100/num, stroke_width=width/num, stroke_color=color,
                                      stroke_opacity=(i-num) ** 2 * 1/num/num, plot_depth=2) for i in range(num+1)]).move_to(self.center)
        self.add(self.shadow)
        return self.shadow

class New_Polygon(VGroup):

    CONFIG = {
        'stroke_color': BLUE,
        'stroke_width': 4,
        'fill_color': BLUE_B,
        'fill_opacity': 0,
    }

    def __init__(self, *vertices, **kwargs):
        VGroup.__init__(self, **kwargs)
        self.lines, self.dots = VGroup(plot_depth=1), VGroup(plot_depth=1)
        self.poly=Polygon(*vertices, fill_color=self.fill_color,
                          fill_opacity=self.fill_opacity, plot_depth=0).set_stroke(width=0)
        self.add(self.poly, self.lines, self.dots)

        n = len(vertices)
        for i in range(n):
            self.lines.add(Line(vertices[i], vertices[(i+1) % n], color=self.stroke_color,
                                stroke_width=self.stroke_width))
            self.dots.add(Dot(vertices[i], color=self.stroke_color).set_height(self.stroke_width/100))

        for dot in self.dots:
            dot.add_updater(lambda d: d.set_height(self.stroke_width/100))

class MySector(VGroup):

    CONFIG = {
        'label': 'label',
        'font': '思源黑体 Bold',
        'value': 1,
    }

    def __init__(self, ):

        pass

class Shadow_2d(VGroup):

    CONFIG = {
        'shadow_color': DARK_BROWN,
        'shadow_opacity': 0.6,
        'blur_width': 0.25,
        'layer_num': 40,
        'scale_factor': 1,
        'shadow_out': True,
        'show_basic_shape': True,
        'plot_depth':-1,
        'rate_func': lambda t: t ** 0.5,
    }

    def __init__(self, mob_or_points, **kwargs):
        VGroup.__init__(self, **kwargs)

        if type(mob_or_points) == list:
            self.shape = Polygon(*mob_or_points, stroke_width=0, plot_depth=-1)
        else:
            self.shape = mob_or_points.set_stroke(width=0)

        self.shape.set_fill(color=self.shadow_color, opacity=self.shadow_opacity * (1 if self.show_basic_shape else 0)).scale(self.scale_factor)
        self.blur_outline = VGroup()
        s = (self.shape.get_height() + self.shape.get_width())/2
        if self.blur_width > 1e-4:
            for i in range(self.layer_num):
                layer_i = self.shape.copy().set_stroke(color=self.shadow_color, width=100 * self.blur_width/self.layer_num, opacity=self.shadow_opacity * (1-self.rate_func(i/self.layer_num))).\
                    set_fill(opacity=0).scale((s + (1 if self.shadow_out else -1) * self.blur_width/self.layer_num * (i+0.5))/ s).set_plot_depth(-2)
                self.blur_outline.add(layer_i)

        self.add(self.shape, self.blur_outline)

class TransformMobject(VGroup):

    CONFIG = {
        'rotate_angle': PI/2,
        'shift_vect': ORIGIN,
        # 'path': ,
        'scale_range': (1, 1e-3),
        'stroke_colors': [RED, PINK, BLUE],
        'num': 10,
        'rate_func': linear,
        'scale_type': 0,

    }

    def __init__(self, mob, **kwargs):

        VGroup.__init__(self, **kwargs)
        if type(self.stroke_colors) == list:
            stroke_colors = color_gradient(self.stroke_colors, self.num)
        else:
            stroke_colors = color_gradient([self.stroke_colors, self.stroke_colors], self.num)
        for i in range(self.num):
            t = i/(self.num-1)
            shift_i = self.rate_func(t) * self.shift_vect
            # scale_i = min(self.scale_range) + self.rate_func(t) * (max(self.scale_range)-min(self.scale_range))
            if self.scale_type == 0:
                scale_i = np.exp(np.log(self.scale_range[0]) + self.rate_func(t) * (np.log(self.scale_range[1])-np.log(self.scale_range[0])))
            else:
                scale_i = self.scale_range[0] + self.rate_func(t) * (self.scale_range[1]-self.scale_range[0])
            theta_i = self.rate_func(t) * self.rotate_angle
            mob_i = mob.copy().shift(shift_i)
            mob_i.scale(scale_i, about_point=mob_i.get_center_of_mass()).rotate(theta_i, about_point=mob_i.get_center_of_mass()).set_stroke(color=stroke_colors[i])
            self.add(mob_i)











"""
关于logo创意：

	1.  基础元素为M和K的负空间设计

	2.  白色部分创意来自弦图

	3.  整体图案本身可生成一个不错的分形

	4.  配色致敬3B1B（具体的蓝色和棕色还得再微调一下）

 logo主要创意由@GrakePCH提供，@GZTime、@cigar666、@鹤翔万里都提供了不少宝贵意见。目前设计工作还在继续完善，希望大家多提意见和建议

"""

from manimlib import *

class Logo(VGroup):

    CONFIG = {
        'color_1': [WHITE, BLUE_B, BLUE_D],
        'color_2': [WHITE, '#C59978', '#8D5630'],

        # 'color_3': [average_color("#CCCCCC", BLUE_C), BLUE_C, BLUE_D],
        # 'color_4': [average_color("#CCCCCC", "#C59978"), '#C59978', '#8D5630'],

        'color_3': [average_color(WHITE, BLUE_C), BLUE_C, BLUE_D],
        'color_4': [average_color(WHITE, "#C59978"), '#C59978', '#8D5630'],

        'center': ORIGIN,
        'size': 2,
        'shift_out': ORIGIN,
        'black_bg': True,
        'add_bg_square': False,
    }

    def __init__(self, **kwargs):
        VGroup.__init__(self, **kwargs)
        self.create_logo()

    def create_logo(self):

        p1 = Polygon(ORIGIN, RIGHT, 2 * UP, stroke_width=0).set_fill(self.color_1[0], 1)
        p2 = Polygon(1.5 * RIGHT, 3 * UR, 3 * UP, stroke_width=0).set_fill(self.color_1[1], 1)
        p3 = Polygon(2 * RIGHT, 3 * RIGHT, 3 * RIGHT + 2 * UP, stroke_width=0).set_fill(self.color_1[2], 1)
        if not self.black_bg:
            p1.set_fill(self.color_3[0], 1), p2.set_fill(self.color_3[1], 1), p3.set_fill(self.color_3[2], 1)

        self.bg = Square(stroke_width=0, fill_color=BLACK if self.black_bg else WHITE, fill_opacity=1).set_height(self.size * 2.5)
        if self.add_bg_square:
            self.add(self.bg)

        self.part_ur = VGroup(p1, p2, p3).move_to([2.5, 1., 0] + self.shift_out)
        self.part_ul = self.part_ur.copy().rotate(PI / 2, about_point=ORIGIN)
        self.part_dl = self.part_ur.copy().rotate(PI, about_point=ORIGIN)
        self.part_dr = self.part_ur.copy().rotate(3 * PI / 2, about_point=ORIGIN)

        self.add(self.part_ur, self.part_ul, self.part_dl, self.part_dr)
        self.set_height(self.size).move_to(self.center)
        if self.black_bg:
            self.part_ur[0].set_fill(self.color_2[0], 1), self.part_ur[1].set_fill(self.color_2[1], 1), self.part_ur[2].set_fill(self.color_2[2], 1)
        else:
            self.part_ur[0].set_fill(self.color_4[0], 1), self.part_ur[1].set_fill(self.color_4[1], 1), self.part_ur[2].set_fill(self.color_4[2], 1)

        self.inner_triangles = VGroup(self.part_ur[0], self.part_ul[0], self.part_dl[0], self.part_dr[0])
        self.mid_triangles = VGroup(self.part_ur[1], self.part_ul[1], self.part_dl[1], self.part_dr[1])
        self.outer_triangles = VGroup(self.part_ur[2], self.part_ul[2], self.part_dl[2], self.part_dr[2])

class Logo_image(Scene):

    CONFIG = {
        'camera_config': {
            'background_color': GREY_A,
        }
    }

    def construct(self):

        logo_black_bg = Logo(size=4.5, add_bg_square=True).shift(LEFT * 3)

        logo_white_bg = Logo(size=4.5, black_bg=False, add_bg_square=True).shift(RIGHT * 3)

        self.add(logo_white_bg, logo_black_bg)

        self.wait(2)

class Logo_01(Scene):

    def construct(self):

        logo = Logo(size=3.2)

        square = VGroup(*[Polygon(ORIGIN, UR, UL), Polygon(ORIGIN, UL, DL), Polygon(ORIGIN, DL, DR), Polygon(ORIGIN, DR, UR),])
        square.set_fill(WHITE, 1).set_stroke(width=0.5, color=WHITE).rotate(np.arctan(0.5)).set_height(logo.inner_triangles.get_height())

        self.add(square)
        self.wait(0.5)
        self.play(ReplacementTransform(square, logo.inner_triangles), run_time=1.5)

        self.wait(0.4)
        self.play(TransformFromCopy(logo.inner_triangles, logo.mid_triangles),
                  TransformFromCopy(logo.inner_triangles, logo.outer_triangles), run_time=2)
        self.wait(0.6)

        big_black_rect = Rectangle(stroke_width=0, fill_color=BLACK, fill_opacity=1).scale(100).align_to(LEFT * 1.4, RIGHT)
        big_black_rect_02 = big_black_rect.copy()
        big_black_rect_02.add_updater(lambda b: b.align_to(logo, RIGHT).shift(RIGHT * 0.15))
        text_font = '思源黑体 Bold'
        text_manim = Text('Manim', font=text_font, size=1.15).align_to(LEFT * 1.4, LEFT).align_to(logo.part_ur, DOWN)
        text_manim.set_color_by_t2c({'M': logo.color_2[2]})
        text_kindergarten = Text('Kindergarten', font=text_font, size=1.15).align_to(logo.part_dr, UP).align_to(text_manim, LEFT)
        text_kindergarten.set_color_by_t2c({'K': logo.color_1[2]})
        text = VGroup(text_manim, text_kindergarten).shift(LEFT * 8).set_plot_depth(-2)
        self.add(text, big_black_rect, big_black_rect_02)
        self.play(logo.shift, LEFT * 3.6, text.shift, RIGHT * 8, run_time=2)

        self.wait(4)

class Logo_02(Scene):
    CONFIG = {
        "font": "Orbitron",
    }

    def construct(self):

        logo = Logo(size=8/3)

        text = VGroup(
            Text("Manim", font=self.font),
            Text("Kindergarten", font=self.font)
        ).arrange(DOWN, aligned_edge=LEFT, buff=0.3).set_height(2).next_to(logo, buff=1.2).shift(DOWN*0.2)
        text[1][0].set_color(logo.color_2[2])
        text[0][0].set_color(logo.color_1[2])
        all_logo = VGroup(logo, text).center()

        line = Line(UP, DOWN, stroke_width=8).move_to(mid(logo.get_right(), text.get_left()))
        line.set_length(1.4)
        text.add(line)

        bg = Rectangle(height=10, width=10, fill_color=BLACK, fill_opacity=1, stroke_width=0)
        bg.add_updater(lambda m: m.move_to(logo, aligned_edge=RIGHT).shift(RIGHT*0.2))

        text.save_state()
        text.shift((text.get_right()[0]-bg.get_right()[0]+0.2)*LEFT)
        logo.save_state()
        logo.move_to(ORIGIN)
        logo.scale(1.2)
        logo.rotate(TAU, axis=IN)
        
        self.add(text, bg)
        self.play(FadeIn(logo[0]))
        self.wait(0.25)
        for i in range(3):
            self.play(MyTransform(logo[i], logo[i+1], about_point=logo.get_center()), run_time=0.2, rate_func=smooth)
        self.wait(0.5)
        self.play(
            text.restore, logo.restore,
            rate_func=smooth, run_time=1
        )
        self.wait()

class Fractal_by_logo(Scene):

    def construct(self):

        logo = Logo(size=5)
        start = logo.part_ur.copy().move_to(ORIGIN).set_height(5)
        to_be_replaced = VGroup(start)

        self.add(start)
        self.wait()
        time_01 = 2.5
        for i in range(5):
            n = len(to_be_replaced)
            to_be_replaced_new = VGroup()
            s = 0
            for m in to_be_replaced:
                to_replace_m = logo.copy().move_to(m.get_center()).set_height(m.get_height() * 1.335)
                if time_01/n < 1/self.camera.frame_rate:
                    self.remove(m)
                    self.add(to_replace_m)
                    s+=1
                    if s *  time_01/n > 1/self.camera.frame_rate:
                        self.wait(s *  time_01 / n)
                        s = 0
                else:
                    self.play(ReplacementTransform(m, to_replace_m), run_time=time_01/n)
                to_be_replaced_new.add(*to_replace_m)
            self.wait(0.8)
            to_be_replaced = to_be_replaced_new
        self.wait(4)

class MyTransform(Animation):
    CONFIG = {
        "radians": PI/2,
        "axis": OUT,
        "about_point": None,
        "remover": True,
    }

    def __init__(self, mobject, target, **kwargs):
        digest_config(self, kwargs)
        self.mobject = mobject.copy()
        self.target = target

    def clean_up_from_scene(self, scene):
        if self.is_remover():
            scene.remove(self.mobject)
            scene.add(self.target)

    def interpolate_mobject(self, alpha):
        now = self.starting_mobject.copy()
        now.rotate(
            alpha * self.radians,
            axis=self.axis,
            about_point=self.about_point,
        )
        for i in range(3):
            now[i].set_color(interpolate_color(self.starting_mobject[i].get_color(), self.target[i].get_color(), alpha))
        self.mobject.become(now)

# final

class Logo_black(Scene):

    CONFIG = {
        "font": "Orbitron",
    }

    def construct(self):

        logo = Logo(size=8/3)
        squares = VGroup(*[Polygon(ORIGIN, UR, UL), Polygon(ORIGIN, UL, DL), Polygon(ORIGIN, DL, DR), Polygon(ORIGIN, DR, UR),])
        squares.set_fill(WHITE, 1).set_stroke(width=0.5, color=WHITE).rotate(np.arctan(0.5)).set_height(logo.inner_triangles.get_height())
        for s in squares:
            s.scale(0.8)

        text = VGroup(
            Text("Manim", font=self.font),
            Text("Kindergarten", font=self.font)
        ).arrange(DOWN, aligned_edge=LEFT, buff=0.3).set_height(2.1).next_to(logo, buff=1.5).shift(DOWN*0.2)
        text[1][0].set_color(logo.color_2[2])
        text[0][0].set_color(logo.color_1[2])
        all_logo = VGroup(logo, text).center()

        line = Line(UP, DOWN, stroke_width=8).move_to(mid(logo.get_right(), text.get_left()))
        line.set_length(1.4)
        text.add(line)

        bg = Rectangle(height=10, width=10, fill_color=BLACK, fill_opacity=1, stroke_width=0)
        bg.add_updater(lambda m: m.move_to(logo, aligned_edge=RIGHT).shift(RIGHT*0.2))

        text.save_state()
        text.shift((text.get_right()[0]-bg.get_right()[0]+0.2)*LEFT)
        logo.save_state()
        logo.move_to(ORIGIN)
        logo.scale(1.5)

        tris = logo.inner_triangles.copy().rotate(-PI)
        self.add(text, bg)

        self.wait(0.3)
        self.add(tris)
        self.wait(0.3)
        self.remove(tris)

        self.wait(0.2)
        self.add(tris)
        self.wait(0.15)
        self.remove(tris)

        self.wait(0.1)
        self.add(tris)
        self.wait(0.1)
        self.remove(tris)
        
        self.wait(0.075)
        self.add(tris)
        self.wait(0.075)
        self.remove(tris)

        self.wait(0.05)
        self.add(tris)
        self.wait(0.05)
        self.remove(tris)
        # square = Square().set_height(tris.get_height()).set_stroke(width=0.5, color=WHITE)
        # self.play(ReplacementTransform(square, tris), run_time=1)
        self.wait(0.2)
        self.play(ShowSubmobjectsOneByOne(tris), rate_func=linear, run_time=0.4)
        for i in tris:
            self.add(i)
            self.wait(0.1)
        self.play(*[ReplacementTransform(tris[i], squares[i]) for i in range(4)], 
            rate_func=rush_from, run_time=0.6)
        #self.play(ReplacementTransform(tris, squares), rate_func=linear, run_time=0.8)
        self.wait(0.1)
        self.play(*[ReplacementTransform(squares[i], logo[i]) for i in range(4)], 
            rate_func=rush_from, run_time=0.6)
        #self.play(ReplacementTransform(squares, logo), rate_func=linear, run_time=1.5)
        self.wait(0.1)
        self.play(
            text.restore, logo.restore,
            rate_func=rush_from, run_time=0.8
        )
        self.wait(1)
        self.play(FadeOut(VGroup(*self.mobjects)))


class Logo_white(Scene):
    CONFIG = {
        "font": "Orbitron",
        "camera_config": {
            "background_color": WHITE,
        },
    }

    def construct(self):

        logo = Logo(size=8/3, black_bg=False)
        squares = VGroup(*[Polygon(ORIGIN, UR, UL), Polygon(ORIGIN, UL, DL), Polygon(ORIGIN, DL, DR), Polygon(ORIGIN, DR, UR),])
        squares.set_fill(BLUE_C, 1).set_stroke(width=0.5, color=BLUE_C).rotate(np.arctan(0.5)).set_height(logo.inner_triangles.get_height())
        squares[0].set_fill('#C59978', 1).set_stroke(width=0.5, color='#C59978')
        for s in squares:
            s.scale(0.8)

        text = VGroup(
            Text("Manim", font=self.font, color=BLACK),
            Text("Kindergarten", font=self.font, color=BLACK)
        ).arrange(DOWN, aligned_edge=LEFT, buff=0.3).set_height(2.1).next_to(logo, buff=1.5).shift(DOWN*0.2)
        text[1][0].set_color(logo.color_2[2])
        text[0][0].set_color(logo.color_1[2])
        all_logo = VGroup(logo, text).center()

        line = Line(UP, DOWN, stroke_width=8, color=BLACK).move_to(mid(logo.get_right(), text.get_left()))
        line.set_length(1.4)
        text.add(line)

        bg = Rectangle(height=10, width=10, fill_color=WHITE, fill_opacity=1, stroke_width=0)
        bg.add_updater(lambda m: m.move_to(logo, aligned_edge=RIGHT).shift(RIGHT*0.2))

        text.save_state()
        text.shift((text.get_right()[0]-bg.get_right()[0]+0.2)*LEFT)
        logo.save_state()
        logo.move_to(ORIGIN)
        logo.scale(1.5)

        tris = logo.inner_triangles.copy().rotate(-PI)
        tris.set_color(BLUE_C)
        tris[0].set_color('#C59978')
        self.add(text, bg)

        self.wait(0.3)
        self.add(tris)
        self.wait(0.3)
        self.remove(tris)

        self.wait(0.2)
        self.add(tris)
        self.wait(0.15)
        self.remove(tris)

        self.wait(0.1)
        self.add(tris)
        self.wait(0.1)
        self.remove(tris)
        
        self.wait(0.075)
        self.add(tris)
        self.wait(0.075)
        self.remove(tris)

        self.wait(0.05)
        self.add(tris)
        self.wait(0.05)
        self.remove(tris)
        # square = Square().set_height(tris.get_height()).set_stroke(width=0.5, color=WHITE)
        # self.play(ReplacementTransform(square, tris), run_time=1)
        self.wait(0.2)
        self.play(ShowSubmobjectsOneByOne(tris), rate_func=linear, run_time=0.4)
        for i in tris:
            self.add(i)
            self.wait(0.1)
        self.play(*[ReplacementTransform(tris[i], squares[i]) for i in range(4)], 
            rate_func=rush_from, run_time=0.6)
        #self.play(ReplacementTransform(tris, squares), rate_func=linear, run_time=0.8)
        self.wait(0.1)
        self.play(*[ReplacementTransform(squares[i], logo[i]) for i in range(4)], 
            rate_func=rush_from, run_time=0.6)
        #self.play(ReplacementTransform(squares, logo), rate_func=linear, run_time=1.5)
        self.wait(0.1)
        self.play(
            text.restore, logo.restore,
            rate_func=rush_from, run_time=0.8
        )
        self.wait(1)
        self.play(FadeOut(VGroup(*self.mobjects)))


class Logo_Rotate_Out(Scene):

    CONFIG = {
        "font": "Orbitron",
    }

    def construct(self):

        logo = Logo(size=8/3)
        squares = VGroup(*[Polygon(ORIGIN, UR, UL), Polygon(ORIGIN, UL, DL), Polygon(ORIGIN, DL, DR), Polygon(ORIGIN, DR, UR),])
        squares.set_fill(WHITE, 1).set_stroke(width=0.5, color=WHITE).rotate(np.arctan(0.5)).set_height(logo.inner_triangles.get_height())
        for s in squares:
            s.scale(0.8)

        text = VGroup(
            VGroup(*[Text(t, font=self.font) for t in 'Manim']).arrange(direction=RIGHT * 0.5, aligned_edge=DOWN),
            VGroup(*[Text(t, font=self.font) for t in 'Kindergarten']).arrange(direction=RIGHT * 0.5, aligned_edge=DOWN)
        ).arrange(DOWN, aligned_edge=LEFT, buff=0.3).set_height(2.1).next_to(logo, buff=1.5).shift(DOWN*0.2)
        text[1][6].align_to(text[1][5], UP)
        text[1][0].set_color(logo.color_2[2])
        text[0][0].set_color(logo.color_1[2])
        all_logo = VGroup(logo, text).center()

        line = Line(UP, DOWN, stroke_width=8).move_to(mid(logo.get_right(), text.get_left()))
        line.set_length(1.4)
        text.add(line)

        bg = Rectangle(height=10, width=10, fill_color=BLACK, fill_opacity=1, stroke_width=0)
        bg.add_updater(lambda m: m.move_to(logo, aligned_edge=RIGHT).shift(RIGHT*0.2))

        text.save_state()
        text.shift((text.get_right()[0]-bg.get_right()[0]+0.2)*LEFT)
        logo.save_state()
        logo.move_to(ORIGIN)
        logo.scale(1.5)

        tris = logo.inner_triangles.copy().rotate(-PI)
        self.add(text, bg)

        self.wait(0.3)
        self.add(tris)
        self.wait(0.3)
        self.remove(tris)

        self.wait(0.2)
        self.add(tris)
        self.wait(0.15)
        self.remove(tris)

        self.wait(0.1)
        self.add(tris)
        self.wait(0.1)
        self.remove(tris)

        self.wait(0.075)
        self.add(tris)
        self.wait(0.075)
        self.remove(tris)

        self.wait(0.05)
        self.add(tris)
        self.wait(0.05)
        self.remove(tris)
        # square = Square().set_height(tris.get_height()).set_stroke(width=0.5, color=WHITE)
        # self.play(ReplacementTransform(square, tris), run_time=1)
        self.wait(0.2)
        self.play(ShowSubmobjectsOneByOne(tris), rate_func=linear, run_time=0.4)
        for i in tris:
            self.add(i)
            self.wait(0.1)
        self.play(*[ReplacementTransform(tris[i], squares[i]) for i in range(4)],
            rate_func=rush_from, run_time=0.6)
        #self.play(ReplacementTransform(tris, squares), rate_func=linear, run_time=0.8)
        self.wait(0.1)
        self.play(*[ReplacementTransform(squares[i], logo[i]) for i in range(4)],
            rate_func=rush_from, run_time=0.6)
        #self.play(ReplacementTransform(squares, logo), rate_func=linear, run_time=1.5)
        self.wait(0.1)
        self.play(
            text.restore, logo.restore,
            rate_func=rush_from, run_time=0.8
        )
        self.wait(0.75)
        self.play(VGroup(*self.mobjects).shift, UP * 1.2)
        self.wait(0.5)
        s = ValueTracker(-7)
        def rotate_out(a, dt):
            w = 1.
            if a.get_center()[0] < s.get_value() or a.get_center()[-1] != 0:
                a.rotate(w * (1 + 2.5 * np.random.random()) * DEGREES, axis=UR, about_point=RIGHT * s.get_value() + OUT * 0.25)
                # a.rotate(w * (1 + 2 * np.random.random()) * DEGREES, axis=UR, about_point=RIGHT * s.get_value() + OUT * (0.12+s.get_value()*0.2))

        for i in range(4):
            for mob in logo[i]:
                mob.add_updater(rotate_out)
        for tex in text[0]:
            tex.add_updater(rotate_out)
        for tex in text[1]:
            tex.add_updater(rotate_out)
        line.add_updater(rotate_out)
        self.remove(bg)
        # s_speed = 0.16
        # for i in range(160):
        #     s.increment_value(s_speed)
        #     self.wait(1/self.camera.frame_rate)
        self.play(s.set_value, 10, rate_func=rush_into, run_time=3.6)

        self.wait(1)




from manimlib import *

class MyBox(Cube):

    CONFIG = {
        'pos': ORIGIN,
        'box_height': 2,
        'bottom_size': [1, 1],
        'fill_opacity': 1,
    }

    def __init__(self, **kwargs):
        Cube.__init__(self, **kwargs)
        self.box_size = np.array([self.bottom_size[0], self.bottom_size[1], self.box_height])
        self.scale(self.box_size/2)
        # self.move_to(self.pos + self.box_height * OUT/2)
        self.move_to(self.pos)
        self.reset_color()

    def update_height(self, new_height):
        self.scale(np.array([1, 1, new_height/self.box_height])) #.shift(OUT * (new_height - self.height)/2)
        self.box_height = new_height

    def update_top_and_bottom(self, top, bottom):
        new_height = abs(top-bottom)
        old_center = self.get_center()
        self.update_height(new_height)
        self.shift(((top+bottom)/2 - old_center[-1]) * OUT)

    def update_top(self, top):
        bottom = self.get_center()[-1] - self.box_height/2
        self.update_top_and_bottom(top, bottom)

    def update_bottom(self, bottom):
        top = self.get_center()[-1] + self.box_height/2
        self.update_top_and_bottom(top, bottom)

    def reset_color(self):
        colors = color_gradient([WHITE, self.get_color(), BLACK], 11)
        self[0].set_fill(color=colors[8])
        self[1].set_fill(color=colors[3])
        self[2].set_fill(color=colors[8])
        self[3].set_fill(color=colors[2])
        self[4].set_fill(color=colors[5])
        self[5].set_fill(color=colors[7])

class MyBoxes(VGroup):

    CONFIG = {
        'center': ORIGIN,
        'bottom_size': [0.25, 0.25],
        'box_height': 2,
        'gap': 0,
        'fill_color': BLUE,
        'resolution': (20, 20),
    }

    def __init__(self, **kwargs):

        VGroup.__init__(self, **kwargs)
        self.create_boxes(self.resolution)
        self.mask_array = np.zeros(self.resolution)
        self.colors = color_gradient([BLUE_D, YELLOW, RED, RED_D], 110)

    def create_boxes(self, resolution=(20, 20)):
        a, b = self.bottom_size[0] + self.gap, self.bottom_size[1] + self.gap
        m, n = resolution[0], resolution[1]
        for i in range(m):
            for j in range(n):
                box_ij = MyBox(pos=a * (j - n/2 + 1/2) * RIGHT + b * (i - m/2 + 1/2) * UP, bottom_size=self.bottom_size,
                               box_height=self.box_height, fill_color=self.fill_color)
                box_ij.reset_color()
                self.add(box_ij)

    def update_height_by_2darray(self, arr_2d):
        m, n = self.resolution[0], self.resolution[1]
        if len(arr_2d)>=m and len(arr_2d[0])>=n:
            for i in range(m):
                for j in range(n):
                    self[i*n+j].update_height(arr_2d[i, j])

    def update_height_by_func(self, func, s=1):
        for box in self:
            center = box.get_center()
            box.update_height(func(center[0], center[1]) * s)

    def update_top_and_bottom_by_2darray(self, arr_top, arr_bottom):
        m, n = self.resolution[0], self.resolution[1]
        if len(arr_top)>=m and len(arr_top[0])>=n and len(arr_bottom)>=m and len(arr_bottom[0])>=n:
            for i in range(m):
                for j in range(n):
                    self[i*n+j].update_top_and_bottom(arr_top[i, j], arr_bottom[i, j])

    def update_top_and_bottom_by_func(self, func_top, func_bottom, s=1):
        for box in self:
            center = box.get_center()
            box.update_top_and_bottom(func_top(center[0], center[1]) * s, func_bottom(center[0], center[1]) * s)

    def update_top_by_func(self, func_top, s=1):
        for box in self:
            center = box.get_center()
            box.update_top(func_top(center[0], center[1]) * s)

    def update_bottom_by_func(self, func_bottom, s=1):
        for box in self:
            center = box.get_center()
            box.update_top(func_bottom(center[0], center[1]) * s)

    def update_color_by_func(self, func):

        a, b = self.bottom_size[0] + self.gap, self.bottom_size[1] + self.gap
        m, n = self.resolution[0], self.resolution[1]
        x, y = np.linspace(-a * n/2, a * n/2, n), np.linspace(-b * m/2, b * m/2, m)
        X, Y = np.meshgrid(x, y)
        Z = func(X, Y)
        z_min, z_max = Z.min(), Z.max()
        # print(z_min, z_max)

        for box in self:
            center = box.get_center() + box.box_height/2 * OUT
            # print(int((func(center[0], center[1]) - z_min)/(z_max-z_min) * 100))
            box.set_color(self.colors[int((func(center[0], center[1]) - z_min)/(z_max-z_min) * 100)])
            box.reset_color()

    def update_color_by_2darray(self, top_array):
        Z = top_array
        m, n = self.resolution[0], self.resolution[1]
        z_min, z_max = Z.min(), Z.max()
        if len(Z) >= m and len(Z) >= n:
            for i in range(m):
                for j in range(n):
                    self[i*n+j].set_color(self.colors[int((Z[i, j] - z_min)/(z_max-z_min) * 100)])
                    self[i*n+j].reset_color()

    def set_mask_array(self, mask):
        self.mask_array = mask

    def divide_by_height(self, h_min=1e-4):
        self.high_boxes, self.short_boxes = VGroup(), VGroup()
        for box in self:
            if box.get_depth() < h_min:
                self.short_boxes.add(box)
            else:
                self.high_boxes.add(box)
        return self.high_boxes, self.short_boxes

    def get_high_boxes(self, h=1e-2):
        return self.divide_by_height(h_min=h)[0]

    def get_short_boxes(self, h=1e-2):
        return self.divide_by_height(h_min=h)[1]

    def apply_mask(self):

        m, n = self.resolution[0], self.resolution[1]
        for i in range(m):
            for j in range(n):
                if self.mask_array[i, j] == 1.: # if self.mask_array[i, j]:
                    self[i*n+j].set_fill(opacity=0)

    def set_mask_by_min_height(self, min_height):

        pass

class Cube_array(VGroup):

    CONFIG = {
        'center': ORIGIN,
        'cube_size': 0.5,
        'gap': 0,
        'fill_color': BLUE,
        'fill_opacity': 1,
        'stroke_color': WHITE,
        'stroke_width': 0,
        'resolution': (4, 4, 1),
        # 'mask_array': None,
        # 'reset_color': True,

    }

    def __init__(self, **kwargs):
        VGroup.__init__(self, **kwargs)
        self.create_cubes()
        self.faces = self.get_all_faces()
        # self.classify_faces()
        self.move_to(self.center)
        self.get_outer_faces()

    def create_cubes(self):
        size, gap = self.cube_size, self.gap
        u, v, w = self.resolution[0], self.resolution[1], self.resolution[2]
        for i in range(u):
            for j in range(v):
                for k in range(w):
                    # box_ijk = MyBox(pos=(size + gap) * ((j - v/2 + 1/2) * RIGHT + (i - u/2 + 1/2) * UP + (k - w/2 + 1/2) * OUT),
                    #                 bottom_size=[size, size], box_height=size, fill_color=self.fill_color,
                    #                 opacity=self.fill_opacity, stroke_color=self.stroke_color, stroke_width=self.stroke_width)
                    box_ijk = Cube(side_length=size, fill_color=self.fill_color, opacity=self.fill_opacity,
                                   stroke_color=self.stroke_color, stroke_width=self.stroke_width)
                    box_ijk.shift((size + gap) * ((j - v/2 + 1/2) * RIGHT + (i - u/2 + 1/2) * UP + (k - w/2 + 1/2) * OUT))

                    # if self.reset_color:
                    #     box_ijk.reset_color()
                    self.add(box_ijk)

    def scale_each_cube(self, scale_factor):
        for cube in self:
            cube.scale(scale_factor)

    def rotate_each_cube(self, angle, axis=OUT, **kwargs):
        for cube in self:
            cube.rotate(angle, axis=OUT, **kwargs)

    def get_all_faces(self):
        faces = VGroup()
        for cube in self:
            faces.add(*cube)
        return faces

    def get_faces_by_range(self, max, min, dim=3):
        max, min = (min, max) if max < min else (max, min)
        faces = VGroup()
        for face in self.faces:
            if face.get_center()[dim-1] <= max and face.get_center()[dim-1] >= min:
                faces.add(face)
        return faces

    def get_top_face(self, err=1e-3):
        a = self.get_zenith()[2]
        self.top_faces = self.get_faces_by_range(a+err, a-err, dim=3)
        return self.top_faces

    def get_bottom_face(self, err=1e-3):
        a = self.get_nadir()[2]
        self.bottom_faces = self.get_faces_by_range(a+err, a-err, dim=3)
        return self.bottom_faces

    def get_front_face(self, err=1e-3):
        a = self.get_bottom()[1]
        self.front_faces = self.get_faces_by_range(a+err, a-err, dim=2)
        return self.front_faces

    def get_back_face(self, err=1e-3):
        a = self.get_top()[1]
        self.back_faces = self.get_faces_by_range(a+err, a-err, dim=2)
        return self.back_faces

    def get_left_face(self, err=1e-3):
        a = self.get_left()[0]
        self.left_faces = self.get_faces_by_range(a+err, a-err, dim=1)
        return self.left_faces

    def get_right_face(self, err=1e-3):
        a = self.get_right()[0]
        self.right_faces = self.get_faces_by_range(a+err, a-err, dim=1)
        return self.right_faces

    def classify_faces(self):

        # max_or_min = np.array([self.get_top()[1], self.get_bottom()[1], self.get_right()[0], self.get_left()[0],
        #               self.get_zenith()[2], self.get_nadir()[2]])
        max_or_min = np.array([self.get_top()[1], self.get_bottom()[1], self.get_right()[0], self.get_left()[0],
                      self.get_zenith()[2], self.get_nadir()[2]])
        print(max_or_min)
        self.outer_faces, self.inner_faces = VGroup(), VGroup()
        err = 1e-4 * self.cube_size ** 2
        for face in self.faces:
            x, y, z = face.get_center()[0], face.get_center()[1], face.get_center()[2]

            a = abs((max_or_min - x) * (max_or_min - y) * (max_or_min - z))
            print('before:', abs(a[0] * a[1] * a[2] * a[3] * a[4] * a[5])/self.cube_size ** 6)
            if abs(a[0] * a[1] * a[2] * a[3] * a[4] * a[5])/(self.cube_size/2) ** 6 < err:
                print('outer:', abs(a[0] * a[1] * a[2] * a[3] * a[4] * a[5])/self.cube_size ** 6)
                self.outer_faces.add(face)
            else:
                print('inner:', abs(a[0] * a[1] * a[2] * a[3] * a[4] * a[5])/self.cube_size ** 6)
                self.inner_faces.add(face)

            # a0 = abs(max_or_min - x)[0] * abs(max_or_min - x)[1]
            # a1 = abs(max_or_min - y)[2] * abs(max_or_min - y)[3]
            # a2 = abs(max_or_min - z)[4] * abs(max_or_min - z)[5]
            # if a0 < err or a1 < err or a2 < err:
            #     self.outer_faces.add(face)
            #     print('outer')
            # else:
            #     print('inner')
            #     self.inner_faces.add(face)
        # return self.outer_faces, self.inner_faces

    def get_outer_faces(self):
        self.outer_faces = VGroup(self.get_top_face(), self.get_bottom_face(),
                                  self.get_front_face(), self.get_back_face(),
                                  self.get_left_face(), self.get_right_face())
        return self.outer_faces

    # def get_innter_faces(self):
    #     return self.inner_faces

class Rubik_Cube(Cube_array):

    CONFIG = {
        'colors': [GREEN, BLUE_D, WHITE, YELLOW, RED_D, ORANGE],
    }

    def __init__(self, size=3, order=3, base_color=GREY_A, **kwargs):
        Cube_array.__init__(self, cube_size=size/order, resolution=(order, order, order), fill_color=base_color, fill_opacity=1, **kwargs)
        self.scale_each_cube(0.9)
        self.order = order
        self.size = size
        self.setup_color()

    def setup_color(self):
        self.top_faces.set_color(self.colors[0])
        self.bottom_faces.set_color(self.colors[1])
        self.front_faces.set_color(self.colors[2])
        self.back_faces.set_color(self.colors[3])
        self.left_faces.set_color(self.colors[4])
        self.right_faces.set_color(self.colors[5])

    def get_layer(self, layer=[1], dim=3):
        faces = VGroup()
        if type(layer) == int:
            a = self.size/2 - 0.5 * self.cube_size - (layer-1) * self.cube_size
            faces.add(self.get_faces_by_range(a + self.cube_size/2, a - self.cube_size/2, dim=dim))

        else:
            for i in layer:
                a = self.size/2 - 0.5 * self.cube_size - (i-1) * self.cube_size
                faces.add(self.get_faces_by_range(a + self.cube_size/2, a - self.cube_size/2, dim=dim))
        return faces

class Rubik_Scene(ThreeDScene):

    CONFIG = {
        'order': 3,
        'size': 4.2,
        'camera_init': {
            'phi': 52.5 * DEGREES,
            'gamma': 0,
            'theta': -45 * DEGREES,
        },
    }

    def setup(self):

        self.set_camera_orientation(**self.camera_init)
        self.rubik = Rubik_Cube(order=self.order, size=self.size)
        self.add(self.rubik)


    def construct(self):

        pass  # To be implemented in subclasses

    def rotate_rubik_anim(self, layer, dim, quarter=1, run_time=1.5, reverse=False, **kwargs):
        theta = quarter * PI/2 * (-1 if reverse else 1)
        axis = [RIGHT, DOWN, OUT][dim-1]
        if layer != 0:
            layer_rotate = [layer] if type(layer) == int else layer
            layer_stay = []
            for i in range(1, self.order+1):
                if not i in layer_rotate:
                    layer_stay.append(i)
            if len(layer_stay) != 0:
                self.play(Rotating(self.rubik.get_layer(layer_rotate, dim=dim), radians=theta, axis=axis, run_time=run_time),
                          Rotating(self.rubik.get_layer(layer_stay, dim=dim), radians=0, axis=axis, run_time=run_time), **kwargs)
            else:
                self.play(Rotating(self.rubik.get_layer(layer_rotate, dim=dim), radians=theta, axis=axis, run_time=run_time), **kwargs)
        else:
            layer_rotate = [i for i in range(1, self.order+1)]
            self.play(Rotating(self.rubik.get_layer(layer_rotate, dim=dim), radians=theta, axis=axis, run_time=run_time), **kwargs)



from manimlib import *

'''
这个文件中的代码是manim交流群的群友提供的，实际上是由@Elteoremadebeethoven(https://github.com/Elteoremadebeethoven)编写的
from @Elteoremadebeethoven(https://github.com/Elteoremadebeethoven)
'''

def return_random_from_word(word):
    """
    This function receives a TextMobject, 
    obtains its length: 
        len(TextMobject("Some text"))
    and returns a random list, example:

    INPUT: word = TextMobjecT("Hello")
    length = len(word) # 4
    rango = list(range(length)) # [0,1,2,3]

    OUTPUT: [3,0,2,1] # Random list
    """
    rango = list(range(len(word)))
    random.shuffle(rango)
    return rango

def return_random_direction(word):
    """
    This function returns a list of random UP or DOWN:
    [UP,UP,DOWN,UP,DOWN,DOWN,...]
    """
    return [random.choice([UP,DOWN]) for _ in range(len(word))]

def get_random_coord(r_x,r_y,step_x,step_y):
    """
    Given two ranges (a, b) and (c, d), this function returns an 
    intermediate array (x, y) such that "x" belongs to (a, c) 
    and "y" belongs to (b, d).
    """
    range_x = list(range(r_x[0],r_x[1],step_x))
    range_y = list(range(r_y[0],r_y[1],step_y))
    select_x = random.choice(range_x)
    select_y = random.choice(range_y)
    return np.array([select_x,select_y,0])

def return_random_coords(word,r_x,r_y,step_x,step_y):
    """
    This function returns a random coordinate array, 
    given the length of a TextMobject
    """
    rango = range(len(word))
    return [word.get_center() + get_random_coord(r_x,r_y,step_x,step_y) for _ in rango]


class WriteRandom(LaggedStart):
    CONFIG = {
        "lag_ratio":0.1,
        "run_time":2.5,
        "anim_kwargs":{},
        "anim_type":Write
    }
    def __init__(self,text,**kwargs):
        digest_config(self, kwargs)
        super().__init__(*[
            self.anim_type(text[i],**self.anim_kwargs)
            for i in return_random_from_word(text)
        ])

class UnWriteRandom(WriteRandom):
    CONFIG = {
        "anim_kwargs": {
            "rate_func": lambda t: smooth(1-t)
        },
        "remover": True,
    }

class FadeInRandom(WriteRandom):
    CONFIG = {
        "anim_type": FadeIn
    }

class FadeOutRandom(WriteRandom):
    CONFIG = {
        "anim_type": FadeOut
    }

class GrowRandom(WriteRandom):
    CONFIG = {
        "anim_type": GrowFromCenter
    }

class UnGrowRandom(GrowRandom):
    CONFIG = {
        "anim_kwargs": {
            "rate_func": lambda t: smooth(1-t),
        },
        "remover": True,
    }

# class FadeInFromRandom(LaggedStart):
#     CONFIG = {
#         "lag_ratio":0.08,
#         "anim_type":FadeIn,
#         "anim_kwargs":{}
#     }
#     def __init__(self,text,**kwargs):
#         digest_config(self, kwargs)
#         super().__init__(*[
#             self.anim_type(text[i],d,**self.anim_kwargs)
#             for i,d in zip(return_random_from_word(text),return_random_direction(text))
#         ])

# class FadeOutFromRandom(FadeInFromRandom):
#     CONFIG = {
#         "anim_type":FadeOutAndShiftDown
#     }

class GrowFromRandom(LaggedStart):
    CONFIG = {
        "lag_ratio":0.2,
        "anim_kwargs":{}
    }
    def __init__(self,text,r_x=[-2,3],r_y=[-2,3],step_x=1,step_y=1,**kwargs):
        digest_config(self, kwargs)
        super().__init__(*[
            GrowFromPoint(text[i],d,**self.anim_kwargs)
            for i,d in zip(return_random_from_word(text),return_random_coords(text,r_x,r_y,step_x,step_y))
        ])

class UnGrowFromRandom(GrowFromRandom):
    CONFIG = {
        "anim_kwargs": {
            "rate_func": lambda t: smooth(1-t)
        },
        "remover": True
    }



from manimlib.constants import *
from manimlib.mobject.types.vectorized_mobject import VGroup
from manimlib.mobject.svg.tex_mobject import Tex, TexText
from manimlib.mobject.svg.text_mobject import Text
from manimlib import *


class MyText(TexMobject):

    CONFIG = {
        'default_font': 'Consolas',
        'tex_scale_factor': 1,
    }

    def __init__(self, *tex_strings, **kwargs):
        self.tex_list = tex_strings
        TexMobject.__init__(self, *tex_strings, **kwargs)
        self.new_font_texs = VGroup()

    def reset_tex_with_font(self):
        self.new_font_texs = VGroup()

    def get_color_by_tex(self, tex, **kwargs):
        parts = self.get_parts_by_tex(tex, **kwargs)
        colors = []
        for part in parts:
            colors.append(part.get_color())
        return colors[0]

    def get_new_font_texs(self, replace_dict):
        for i in range(len(self.tex_strings)):
            tex = self.tex_strings[i]
            color=self.get_color_by_tex(tex)
            if tex in replace_dict:
                tex = replace_dict[tex]
            tex_new = Text(tex, font=self.default_font, color=color)
            tex_new.set_height(self[i].get_height())
            # if tex == '-' or tex == '=':
            #     tex_new.set_width(self[i].get_width(), stretch=True)
            if tex == '-':
                tex_new.set_width(self[i].get_width(), stretch=True).scale([0.8, 2.4, 1])
            elif tex == '+':
                tex_new.set_width(self[i].get_width(), stretch=True).scale(0.8)
            elif tex == '×':
                tex_new.set_width(self[i].get_width(), stretch=True).scale(0.9)
            elif tex == '=':
                tex_new.set_width(self[i].get_width(), stretch=True).scale([0.9, 1.35, 1])

            tex_new.scale(self.tex_scale_factor)
            tex_new.move_to(self[i])
            if tex == '-':
                tex_new.shift(UP * 0.5 * tex_new.get_height())
            elif tex == '=':
                tex_new.shift(UP * 0.15 * tex_new.get_height())
            self.new_font_texs.add(tex_new)
        return self.new_font_texs

# class PStyle_Text(VGroup):
#
#     CONFIG = {
#         'size': 1,
#         'colors': [BLACK, WHITE, ORANGE],
#         'font': '思源黑体 Bold',
#     }
#
#     def __init__(self, tex_1, tex_2, **kwargs):
#
#         VGroup.__init__(self, **kwargs)
#         self.text_1 = Text(tex_1, font=self.font, color=self.colors[1], plot_depth=0.2, size=self.size * 0.64)
#         self.text_2 = Text(tex_2, font=self.font, color=self.colors[0], plot_depth=0.2, size=self.size * 0.56)
#         # self.text_2.shift(RIGHT * (self.size * 0.3 + self.text_1.get_width()/2 + self.text_2.get_width()/2))
#         self.text_2.next_to(self.text_1, RIGHT * self.size * 0.32, buff=1)
#         self.bg_2 = SurroundingRectangle(self.text_2, stroke_width=0, fill_color=self.colors[2], fill_opacity=1, plot_depth=0.1).scale([1.02, 1.15, 1]).round_corners(self.size * 0.1)
#         self.bg_all = SurroundingRectangle(VGroup(self.text_1, self.text_1, self.bg_2), stroke_width=0, fill_color=self.colors[0], fill_opacity=1, plot_depth=0).scale(1.15)
#         self.add(self.bg_all, self.text_1, VGroup(self.bg_2, self.text_2))

# def flat(nestedlist):
#     outcome = [nestedlist[i][j] for i in range(len(nestedlist)) for j in range(len(nestedlist[i]))]
#     return outcome
#
# class Test01(Scene):
#
#     def construct(self):
#
#         t = TexMobject('12', '3', '456').to_edge(UP)
#         t_sub = VGroup(*flat(t)).to_edge(UP * 3)
#         for tex in t:
#             self.play(Write(tex))
#         self.wait()
#         for tex in t_sub:
#             self.play(Write(tex))
#         self.wait()

class Text4anim(VGroup):

    CONFIG = {
        'rate_func': there_and_back,
        'proportion': 0.4,
        'shift_vect': UP,
        'scale_factor': 1,
        # 'rotate_angle': 0,
        'change_opacity': False,
        'init_opacity': 1,
        'reverse_order': False,
    }

    def __init__(self, texmob, **kwargs):

        self.texmob = texmob
        self.anim_controller = ValueTracker(0.)
        VGroup.__init__(self, *[self.texmob[i][j] for i in range(len(self.texmob)) for j in range(len(self.texmob[i]))], **kwargs)
        self.submob_pos = [mob.get_center() for mob in self]
        self.submob_size = [mob.get_height() for mob in self]
        if self.change_opacity:
            self.submob_opacity = [self.init_opacity for mob in self]
        else:
            self.submob_opacity = [1 for mob in self]
        # self.get_new_ratefunc()
        self.activate_anim()

    def shift_(self, *vectors):
        self.shift(*vectors)
        for pos in self.submob_pos:
            pos += reduce(op.add, vectors)

    def anim_updater(self, texmob, dt):
        n = len(texmob)
        for i in range(n):
            t = i/n
            t_i = self.rate_func((self.anim_controller.get_value() * (1.02 + self.proportion) - t)/self.proportion)

            if not self.reverse_order:
                texmob[i].move_to(self.submob_pos[i] + self.shift_vect * t_i)\
                    .set_height(self.submob_size[i] * (1 + (self.scale_factor - 1) * t_i))
                if self.change_opacity:
                    opacity_i = self.submob_opacity[i] + t_i
                    texmob[i].set_opacity(opacity_i if opacity_i < 1 else 1)
            else:
                texmob[n-1-i].move_to(self.submob_pos[n-1-i] + self.shift_vect * t_i)\
                    .set_height(self.submob_size[n-1-i] * (1 + (self.scale_factor - 1) * t_i))
                if self.change_opacity:
                    opacity_i = self.submob_opacity[n-1-i] + t_i
                    texmob[n-1-i].set_opacity(opacity_i if opacity_i < 1 else 1)

    def activate_anim(self):
        self.add_updater(self.anim_updater)

class Test_anim(Scene):

    def construct(self):

        # tex_mob = Text('xgnb xgnb xgnb xgnb xgnb xgnb', font='庞门正道标题体').set_height(0.64)
        tex_mob = TexMobject('\\sum i^3', '=', '1^3', '+', '2^3', '+', '3^3', '+', '4^3', '+', '\\cdots', '+', 'n^3').set_height(0.9)

        tex_anim_mob = Text4anim(tex_mob, proportion=0.6)

        self.add(tex_anim_mob)

        self.wait()
        self.play(tex_anim_mob.anim_controller.set_value, 1, rate_func=there_and_back, run_time=5.6)
        self.wait()
        tex_anim_mob.rate_func = smooth
        self.play(tex_anim_mob.anim_controller.set_value, 1, rate_func=smooth, run_time=2.5)
        self.wait()

        tex_anim_mob.proportion = 0.45
        tex_anim_mob.anim_controller.set_value(0)
        tex_anim_mob.shift_(UP)

        tex_anim_mob.shift_vect = DOWN
        self.play(tex_anim_mob.anim_controller.set_value, 1, rate_func=smooth, run_time=2.5)
        self.wait()

        tex_anim_mob.anim_controller.set_value(0)
        tex_anim_mob.shift_(DOWN)

        tex_anim_mob.rate_func = there_and_back
        tex_anim_mob.shift_vect = ORIGIN
        tex_anim_mob.scale_factor = 1.6
        self.play(tex_anim_mob.anim_controller.set_value, 1, rate_func=smooth, run_time=2.5)
        self.wait()

        tex_anim_mob.anim_controller.set_value(0)
        tex_anim_mob.proportion = 0.75
        tex_anim_mob.rate_func = there_and_back
        tex_anim_mob.shift_vect = UP * 1.6
        tex_anim_mob.scale_factor = 2.4
        self.play(tex_anim_mob.anim_controller.set_value, 1, rate_func=smooth, run_time=2.75)

        self.wait(2)

class Text4animScene(Scene):

    def ShiftInOneByOne(self, text, shift_vect=LEFT * 1.5, run_time=2, wait_time=1, stop_updater=True):
        text.shift(-shift_vect)
        t4a = Text4anim(text, shift_vect=shift_vect, rate_func=smooth, proportion=0.32, change_opacity=True, init_opacity=0)
        self.add(t4a)
        self.play(t4a.anim_controller.set_value, 1, run_time=run_time)
        if stop_updater:
            t4a.suspend_updating(t4a.anim_updater)
        self.wait(wait_time)
        return t4a

    def ShiftInOneByOne_new(self, text, shift_vect=LEFT * 1.5, run_speed=0.24, wait_time=1, rate_func=smooth, stop_updater=True):
        text.shift(-shift_vect)
        t4a = Text4anim(text, shift_vect=shift_vect, rate_func=smooth, proportion=0.45, change_opacity=True, init_opacity=0)
        self.add(t4a)
        self.play(t4a.anim_controller.set_value, 1, run_time=run_speed * len(t4a) + 0.05, rate_func=rate_func)
        if stop_updater:
            # t4a.suspend_updating(t4a.anim_updater)
            t4a.clear_updaters()
        self.wait(wait_time)
        return t4a

    def Countdown_anim(self, time=5):

        num = VGroup(*[Text(str(i), font='思源黑体 Bold', color=GREEN).set_height(1) for i in range(time, 0, -1)]).to_corner(RIGHT * 1.2 + UP * 1.2, buff=1)
        circle = Circle(radius=0.9).move_to(num).set_stroke(GREEN, 12)
        for i in range(time):
            self.add(num[i])
            self.play(WiggleOutThenIn(num[i]), ShowCreationThenDestruction(circle), run_time=1)
            self.remove(num[i])

class MyTitle(Text):

    CONFIG = {
        "scale_factor": 1,
        "include_underline": True,
        "underline_width": FRAME_WIDTH - 2,
        # This will override underline_width
        "match_underline_width_to_text": False,
        "underline_buff": MED_SMALL_BUFF,
    }

    def __init__(self, *text, **kwargs):
        Text.__init__(self, *text, **kwargs)
        self.scale(self.scale_factor)
        self.to_edge(UP)
        if self.include_underline:
            underline = Line(LEFT, RIGHT)
            underline.next_to(self, DOWN, buff=self.underline_buff)
            if self.match_underline_width_to_text:
                underline.match_width(self)
            else:
                underline.set_width(self.underline_width)
            self.add(underline)
            self.underline = underline



import numpy as np
from manimlib import *
from .quaternion import Quaternion


class Vec3:
    def __init__(self, x=None, y=1, z=1):
        if issubclass(type(x), (np.ndarray, list, tuple)):
            self._x = x[0]
            self._y = x[1]
            self._z = x[2]
        else:
            if x is None:
                x = 1
            self._x = x
            self._y = y
            self._z = z

        self._vector = None
        self._set_vec()

    @property
    def x(self):
        return self._x

    @property
    def y(self):
        return self._y

    @property
    def z(self):
        return self._z

    def _set_vec(self):
        self._vector = np.array([self._x, self._y, self._z])

    def apply_quaternion(self, q: Quaternion):
        x, y, z = self._x, self._y, self._z
        # calculate quat * vector
        ix = q.w * x + q.y * z - q.z * y
        iy = q.w * y + q.z * x - q.x * z
        iz = q.w * z + q.x * y - q.y * x
        iw = -q.x * x - q.y * y - q.z * z

        # calculate result * inverse quat
        self._x = ix * q.w + iw * -q.x + iy * -q.z - iz * -q.y
        self._y = iy * q.w + iw * -q.y + iz * -q.x - ix * -q.z
        self._z = iz * q.w + iw * -q.z + ix * -q.y - iy * -q.x

        self._set_vec()
        return self

    def dot(self, vec):
        self._vector.dot(vec)

    def to_array(self):
        return self._vector

    def copy(self):
        obj = object.__new__(Vec3)
        obj.__init__(*self._q)
        return obj

    def __str__(self):
        return self._vector.__str__()


if __name__ == "__main__":
    vec_list = [2, 1, 1]

    vec3 = Vec3(*vec_list)
    m_vec = Arrow(ORIGIN, vec_list, buff=0)

    axis = [1, 2, 1]
    angle = 45 * DEGREES

    m_vec.rotate(angle, axis)

    # print(quat_conjugate(np.array(Q), style=1))
    # print(quaternion_conjugate(Q))
    q = Quaternion().set_from_axis_angle([1, 1, 1], 45 * DEGREES)
    print(q, 'q')
    print(vec3.apply_quaternion(q))
    print(m_vec.get_end())



#!/usr/bin/env python

from __future__ import print_function
from setuptools import setup, find_packages
from glob import glob
import os
from sparrow.file_ops import yaml_load


pkgname = "manim_express"
pkgdir = pkgname
version_config = yaml_load(os.path.join(pkgdir, "version-config.yaml"))
name, version = version_config['name'], version_config['version']


with open(glob('requirements.txt')[0], encoding='utf-8') as f:
    all_reqs = f.read().split('\n')
install_requires = [x.strip() for x in all_reqs]

with open("README.md", "r", encoding='utf-8') as fr:
    long_description = fr.read()

setup(name=name,
      version=version,
      package_data={
          pkgdir: [
              '*.yaml', '*.yml',
          ],
      },
      description=" Animation engine for explanatory math videos",
      long_description=long_description,
      long_description_content_type="text/markdown",
      author="K.y",
      author_email="beidongjiedeguang@gmail.com",
      url="https://github.com/beidongjiedeguang/manim-express",
      license="MIT",
      install_requires=install_requires,
      classifiers=[
          'Operating System :: OS Independent',
          'Intended Audience :: Developers',
          'Intended Audience :: Science/Research',
          'Topic :: Scientific/Engineering :: Artificial Intelligence',
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.8",
      ],
      keywords=[
          'Computer Vision', 'Mathematics', 'Physics', 'Machine Learning', 'Neural Networks',
      ],
      packages=find_packages()
      )




from manimlib.utils.tex_file_writing import tex_hash
from manimlib.utils.directories import get_tex_dir
import requests
import urllib
from fake_headers import Headers
import os
import re
# from helium import *
# import pyperclip


def tex_to_svg_file_online(tex_file_content):
    """ Using the online latex compiler: https://www.quicklatex.com/latex3.f
    https://latexonline.cc/
    """
    svg_file_path = os.path.join(
        get_tex_dir(), tex_hash(tex_file_content) + ".svg"
    )
    tex_file = svg_file_path.replace(".svg", ".tex")

    if os.path.exists(svg_file_path):
        pass
    else:
        idx = re.search("begin{document}", tex_file_content, flags=0).span()[0] - 2
        preamble, expression = tex_file_content[:idx], tex_file_content[idx:]

        expression = expression.\
            replace("\\begin{document}", '').\
            replace("\\end{document}", '').\
            strip('\n')

        with open(tex_file, "w", encoding="utf-8") as outfile:
            outfile.write(tex_file_content)

        def quick_latex():
            params = {
                "formula": expression,
                "preamble": preamble,
                "out": 2,  # 2 For SVG output
            }
            payload = urllib.parse.urlencode(
                params, quote_via=urllib.parse.quote
            )
            response = requests.post("https://www.quicklatex.com/latex3.f", data=payload)

            if not response.text.startswith("0"):
                error = "\n".join(
                    response.text.split("\r\n")[2:]
                )
                raise ResourceWarning(error)
            else:
                svgurl = response.text.split("\n")[-1].split(" ")[0].replace("png", "svg")
                headers = Headers(os="mac", headers=True).generate()
                content = requests.get(svgurl, headers=headers).text
                return content

        # def chrome_helium():
        #     url = "https://viereck.ch/latex-to-svg/"
        #     driver = start_chrome(url, headless=False)
        #     e1 = driver.find_element_by_tag_name("textarea")
        #     write(expression, into=e1)
        #     #     driver.find_element_by_id("environment") # to choice align? It doesn't seem necessary..
        #     e2 = driver.find_element_by_id("svgSourceCodeButton")
        #     e2.click()
        #     e3 = driver.find_element_by_id("svgSourceCode")
        #     e3.send_keys(Keys.CONTROL, 'a')
        #     e3.send_keys(Keys.CONTROL, 'c')
        #     content = pyperclip.paste()
        #     return content

        # svg_content = chrome_helium()
        svg_content = quick_latex()
        with open(svg_file_path, "w", encoding="utf-8") as fo:
            fo.write(svg_content)

    # Remove tex files (not svg files)
    tex_dir, name = os.path.split(svg_file_path)
    stem, end = name.split(".")
    for file in filter(lambda s: s.startswith(stem), os.listdir(tex_dir)):
        if not file.endswith(end):
            os.remove(os.path.join(tex_dir, file))

    return svg_file_path
